import os
import platform
import textwrap

import pytest

from conan.api.model import RecipeReference
from conan.test.utils.tools import TestClient


@pytest.mark.skipif(platform.system() == "Windows", reason="symlink need admin privileges")
class TestFixSymlinks:
    name_ref = RecipeReference.loads("name/version")

    conanfile = textwrap.dedent("""
        import os
        import shutil
        from conan import ConanFile
        from conan.tools.files import save
        from conan.tools.files.symlinks import (absolute_to_relative_symlinks,
                                                remove_external_symlinks,
                                                remove_broken_symlinks)

        class Recipe(ConanFile):

            def build(self):
                save(self, os.path.join(self.build_folder, "build.txt"), "contents")

            def package(self):
                os.symlink('/dev/null', os.path.join(self.package_folder, "black_hole"))

                # Files: Symlink to file outside the package
                os.symlink(os.path.join(self.build_folder, "build.txt"),
                           os.path.join(self.package_folder, "outside_symlink.txt"))

                # Files: A regular file with symlinks to it
                save(self, os.path.join(self.package_folder, "regular.txt"), "contents")
                os.symlink(os.path.join(self.package_folder, "regular.txt"),
                           os.path.join(self.package_folder, "absolute_symlink.txt"))
                os.symlink("regular.txt", os.path.join(self.package_folder, "relative_symlink.txt"))

                # Files: A broken symlink
                save(self, os.path.join(self.package_folder, "file.txt"), "contents")
                os.symlink(os.path.join(self.package_folder, "file.txt"),
                           os.path.join(self.package_folder, "broken.txt"))
                os.unlink(os.path.join(self.package_folder, "file.txt"))

                # Folder: Symlink outside package
                os.symlink(self.build_folder, os.path.join(self.package_folder, "outside_folder"))

                # Folder: a regular folder and symlinks to it
                save(self, os.path.join(self.package_folder, "folder", "file.txt"), "contents")
                os.symlink(os.path.join(self.package_folder, "folder"),
                           os.path.join(self.package_folder, "absolute"))
                os.symlink("folder", os.path.join(self.package_folder, "relative"))

                # Folder: broken symlink
                save(self, os.path.join(self.package_folder, "tmp", "file.txt"), "contents")
                os.symlink(os.path.join(self.package_folder, "tmp"),
                           os.path.join(self.package_folder, "broken_folder"))
                shutil.rmtree(os.path.join(self.package_folder, "tmp"))

                #
                os.symlink(os.path.join(self.package_folder, "folder", "file.txt"),
                           os.path.join(self.package_folder, "abs_to_file_in_folder.txt"))

                # --> Run the tools
                absolute_to_relative_symlinks(self, self.package_folder)
                remove_external_symlinks(self, self.package_folder)
                remove_broken_symlinks(self, self.package_folder)
    """)

    def test_error_reported(self):
        t = TestClient()
        t.save({'conanfile.py': self.conanfile})
        t.run("create . --name=name --version=version")
        package_folder = t.created_layout().package()

        assert sorted(os.listdir(package_folder)) == ['abs_to_file_in_folder.txt',
                                                      'absolute',
                                                      'absolute_symlink.txt',
                                                      'conaninfo.txt',
                                                      'conanmanifest.txt',
                                                      'folder',
                                                      'regular.txt',
                                                      'relative',
                                                      'relative_symlink.txt']
        # All the links in the package_folder are relative and contained into it
        for (dirpath, dirnames, filenames) in os.walk(package_folder):
            for filename in filenames:
                filename = os.path.join(dirpath, filename)
                if os.path.islink(filename):
                    rel_path = str(os.readlink(filename))
                    assert not os.path.exists(os.path.abspath(rel_path))
                    assert not rel_path.startswith('..')
            for dirname in dirnames:
                dirname = os.path.join(dirpath, dirname)
                if os.path.islink(dirname):
                    rel_path = str(os.readlink(dirname))
                    assert not os.path.exists(os.path.abspath(rel_path))
                    assert not rel_path.startswith('..')



import os
import textwrap
import zipfile

from conan.tools.files import unzip
from conan.test.utils.mocks import RedirectedTestOutput, ConanFileMock
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient, redirect_output
from conan.internal.util.files import load, save


class TestOutput:

    def test_error(self):
        client = TestClient()
        conanfile = textwrap.dedent("""
            from conan import ConanFile

            class PkgConan(ConanFile):
                def source(self):
                   self.output.info("TEXT ÑÜíóúéáàèòù абвгдежзийкл 做戏之说  ENDTEXT")
            """)
        client.save({"conanfile.py": conanfile})
        client.run("install .")
        client.run("source .")
        assert "TEXT" in client.out
        assert "ENDTEXT" in client.out

    def test_unzip_output(self):
        tmp_dir = temp_folder()
        file_path = os.path.join(tmp_dir, "example.txt")
        save(file_path, "Hello world!")

        zip_path = os.path.join(tmp_dir, 'example.zip')
        zipf = zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED)
        for root, _, files in os.walk(tmp_dir):
            for f in files:
                zipf.write(os.path.join(root, f), f)
        zipf.close()

        output_dir = os.path.join(tmp_dir, "output_dir")
        captured_output = RedirectedTestOutput()
        with redirect_output(captured_output):
            unzip(ConanFileMock(), zip_path, output_dir)

        output = captured_output.getvalue()
        assert "Unzipping" in output
        content = load(os.path.join(output_dir, "example.txt"))
        assert content == "Hello world!"



import os

from conan.tools.files import rename
from conan.test.utils.tools import TestClient
from conan.internal.util.files import chdir


class TestRename:
    def verify_dir(self, basedir):
        assert os.path.isdir(basedir)

        assert os.path.isfile(os.path.join(basedir, "1.txt"))
        assert os.path.isfile(os.path.join(basedir, "1.pdb"))
        assert os.path.isfile(os.path.join(basedir, "1.pdb1"))

        assert os.path.isdir(os.path.join(basedir, "dir.pdb"))
        assert os.path.isfile(os.path.join(basedir, "dir.pdb", "2.txt"))
        assert os.path.isfile(os.path.join(basedir, "dir.pdb", "2.pdb"))
        assert os.path.isfile(os.path.join(basedir, "dir.pdb", "2.pdb1"))

        assert os.path.isdir(os.path.join(basedir, "middir"))
        assert os.path.isfile(os.path.join(basedir, "middir", "3.txt"))
        assert os.path.isfile(os.path.join(basedir, "middir", "3.pdb"))
        assert os.path.isfile(os.path.join(basedir, "middir", "3.pdb1"))

        assert os.path.isdir(os.path.join(basedir, "middir", "deepdir"))
        assert os.path.isfile(os.path.join(basedir, "middir", "deepdir", "4.txt"))
        assert os.path.isfile(os.path.join(basedir, "middir", "deepdir", "4.pdb"))
        assert os.path.isfile(os.path.join(basedir, "middir", "deepdir", "4.pdb1"))

    def test_rename(self):
        client = TestClient()
        tmpdir = client.current_folder

        sub_space_dir = "sub dir"
        with chdir(tmpdir):
            os.makedirs(sub_space_dir)
            os.makedirs(os.path.join(sub_space_dir, "dir.pdb"))
            os.makedirs(os.path.join(sub_space_dir, "middir"))
            os.makedirs(os.path.join(sub_space_dir, "middir", "deepdir"))

        client.save({os.path.join(sub_space_dir, "1.txt"): "",
                     os.path.join(sub_space_dir, "1.pdb"): "",
                     os.path.join(sub_space_dir, "1.pdb1"): "",
                     os.path.join(sub_space_dir, "dir.pdb", "2.txt"): "",
                     os.path.join(sub_space_dir, "dir.pdb", "2.pdb"): "",
                     os.path.join(sub_space_dir, "dir.pdb", "2.pdb1"): "",
                     os.path.join(sub_space_dir, "middir", "3.txt"): "",
                     os.path.join(sub_space_dir, "middir", "3.pdb"): "",
                     os.path.join(sub_space_dir, "middir", "3.pdb1"): "",
                     os.path.join(sub_space_dir, "middir", "deepdir", "4.txt"): "",
                     os.path.join(sub_space_dir, "middir", "deepdir", "4.pdb"): "",
                     os.path.join(sub_space_dir, "middir", "deepdir", "4.pdb1"): ""
                    })
        self.verify_dir(os.path.join(tmpdir, sub_space_dir))

        with chdir(tmpdir):
            rename(None, sub_space_dir, "dst dir")
            self.verify_dir(os.path.join(tmpdir, "dst dir"))

            rename(None,"dst dir", "subdir")
            self.verify_dir(os.path.join(tmpdir, "subdir"))

            rename(None, os.path.join("subdir", "1.txt"), "t.txt")
            assert os.path.isfile(os.path.join(tmpdir, "t.txt"))
            assert not os.path.isfile(os.path.join(tmpdir, "subdir", "1.txt"))

    def test_rename_empty_folder(self):
        client = TestClient()
        old_folder = os.path.join(client.current_folder, "old_folder")
        os.mkdir(old_folder)
        new_folder = os.path.join(client.current_folder, "new_folder")
        rename(None, old_folder, new_folder)
        assert not os.path.exists(old_folder)
        assert os.path.exists(new_folder)



import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.tool("cmake")
def test_conan_new_compiles():
    # TODO: Maybe add more templates that are not used in the rest of the test suite?
    tc = TestClient()
    tc.run("new header_lib -d name=hello -d version=1.0 -o=hello")
    tc.run("new header_lib -d name=bye -d version=1.0 -d requires=hello/1.0 -o=bye")

    tc.run("create hello -tf=")
    tc.run("create bye")



import textwrap

import pytest

from parameterized import parameterized

from conan.internal.graph.graph_error import GraphConflictError, GraphLoopError
from conan.api.model import RecipeReference
from test.integration.graph.core.graph_manager_base import GraphManagerTest
from conan.test.utils.tools import GenConanfile, NO_SETTINGS_PACKAGE_ID, TestClient


def _check_transitive(node, transitive_deps):
    values = list(node.transitive_deps.values())

    assert len(values) == len(transitive_deps), f"{node}:{len(values)} != {len(transitive_deps)}"

    for v1, v2 in zip(values, transitive_deps):
        # asserts were difficult to debug
        if v1.node is not v2[0]: raise Exception(f"{v1.node}!={v2[0]}")
        if v1.require.headers is not v2[1]: raise Exception(f"{v1.node}!={v2[0]} headers")
        if v1.require.libs is not v2[2]: raise Exception(f"{v1.node}!={v2[0]} libs")
        if v1.require.build is not v2[3]: raise Exception(f"{v1.node}!={v2[0]} build")
        if v1.require.run is not v2[4]: raise Exception(f"{v1.node}!={v2[0]} run")
        if len(v2) >= 6:
            if v1.require.test is not v2[5]: raise Exception(f"{v1.node}!={v2[0]} test")


class TestBuildRequiresGraph(GraphManagerTest):

    @parameterized.expand([("recipe", ), ("profile", )])
    def test_basic(self, build_require):
        # app -(br)-> cmake
        self._cache_recipe("cmake/0.1", GenConanfile())
        if build_require == "recipe":
            profile_build_requires = None
            conanfile = GenConanfile("app", "0.1").with_tool_requires("cmake/0.1")
        else:
            profile_build_requires = {"*": [RecipeReference.loads("cmake/0.1")]}
            conanfile = GenConanfile("app", "0.1")

        deps_graph = self.build_graph(conanfile, profile_build_requires=profile_build_requires,
                                      install=False)

        # Build requires always apply to the consumer
        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        cmake = app.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[cmake], dependents=[])
        self._check_node(cmake, "cmake/0.1#123", deps=[], dependents=[app])

    def test_lib_build_require(self):
        # app -> lib -(br)-> cmake
        self._cache_recipe("cmake/0.1", GenConanfile())
        self._cache_recipe("lib/0.1", GenConanfile().with_tool_requires("cmake/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake], dependents=[app])
        self._check_node(cmake, "cmake/0.1#123", deps=[], dependents=[lib])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])
        _check_transitive(lib, [(cmake, False, False, True, True)])

    @parameterized.expand([("shared", ), ("static", ), ("notrun", ), ("run", )])
    def test_build_require_transitive(self, cmakelib_type):
        # app -> lib -(br)-> cmake -> cmakelib (cmakelib_type)

        if cmakelib_type in ("notrun", "run"):  # Unknown
            cmakelib = GenConanfile().with_settings("os")
        else:
            cmakelib = GenConanfile().with_settings("os").\
                with_shared_option(cmakelib_type == "shared")
        run = True if cmakelib_type == "run" else None  # Not necessary to specify

        self._cache_recipe("cmakelib/0.1", cmakelib)
        self._cache_recipe("cmake/0.1", GenConanfile().with_settings("os").
                           with_requirement("cmakelib/0.1", run=run))
        self._cache_recipe("lib/0.1", GenConanfile().with_settings("os").
                           with_tool_requires("cmake/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_settings("os").
                                      with_require("lib/0.1"))

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst
        cmakelib = cmake.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[], settings={"os": "Linux"})
        self._check_node(lib, "lib/0.1#123", deps=[cmake], dependents=[app],
                         settings={"os": "Linux"})
        self._check_node(cmake, "cmake/0.1#123", deps=[cmakelib], dependents=[lib],
                         settings={"os": "Windows"})
        self._check_node(cmakelib, "cmakelib/0.1#123", deps=[], dependents=[cmake],
                         settings={"os": "Windows"})

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])

        if cmakelib_type in ("static", "notrun"):
            _check_transitive(lib, [(cmake, False, False, True, True)])
        else:
            _check_transitive(lib, [(cmake, False, False, True, True),
                                    (cmakelib, False, False, True, True)])

    def test_build_require_bootstrap(self):
        # app -> lib -(br)-> cmake/2 -(br)-> cmake/1
        self._cache_recipe("cmake/0.1", GenConanfile())
        self._cache_recipe("cmake/0.2", GenConanfile().with_tool_requires("cmake/0.1"))
        self._cache_recipe("lib/0.1", GenConanfile().with_tool_requires("cmake/0.2"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake2 = lib.edges[0].dst
        cmake1 = cmake2.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake2], dependents=[app])
        self._check_node(cmake2, "cmake/0.2#123", deps=[cmake1], dependents=[lib])
        self._check_node(cmake1, "cmake/0.1#123", deps=[], dependents=[cmake2])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])
        _check_transitive(lib, [(cmake2, False, False, True, True)])
        _check_transitive(cmake2, [(cmake1, False, False, True, True)])

    def test_build_require_private(self):
        # app -> lib -(br)-> cmake -(private)-> zlib
        self._cache_recipe("zlib/0.1", GenConanfile())
        self._cache_recipe("cmake/0.1", GenConanfile().with_requirement("zlib/0.1", visible=False))
        self._cache_recipe("lib/0.1", GenConanfile().with_tool_requires("cmake/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst
        zlib = cmake.edges[0].dst

        self._check_node(app, "app/0.1", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake], dependents=[app])
        self._check_node(cmake, "cmake/0.1#123", deps=[zlib], dependents=[lib])
        self._check_node(zlib, "zlib/0.1#123", deps=[], dependents=[cmake])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])
        _check_transitive(lib, [(cmake, False, False, True, True)])
        _check_transitive(cmake, [(zlib, True, True, False, False)])


class TestBuildRequiresTransitivityDiamond(GraphManagerTest):

    def test_build_require_transitive_static(self):
        # app -> lib -(br)-> cmake -> zlib1 (static)
        #          \--(br)-> mingw -> zlib2 (static)
        self._cache_recipe("zlib/0.1", GenConanfile().with_shared_option(False))
        self._cache_recipe("zlib/0.2", GenConanfile().with_shared_option(False))
        self._cache_recipe("cmake/0.1", GenConanfile().with_require("zlib/0.1"))
        self._cache_recipe("mingw/0.1", GenConanfile().with_require("zlib/0.2"))
        self._cache_recipe("lib/0.1", GenConanfile().with_tool_requires("cmake/0.1",
                                                                              "mingw/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 6 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst
        mingw = lib.edges[1].dst
        zlib1 = cmake.edges[0].dst
        zlib2 = mingw.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake, mingw], dependents=[app])
        self._check_node(cmake, "cmake/0.1#123", deps=[zlib1], dependents=[lib])
        self._check_node(zlib1, "zlib/0.1#123", deps=[], dependents=[cmake])
        self._check_node(mingw, "mingw/0.1#123", deps=[zlib2], dependents=[lib])
        self._check_node(zlib2, "zlib/0.2#123", deps=[], dependents=[mingw])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])
        _check_transitive(lib, [(cmake, False, False, True, True),
                                (mingw, False, False, True, True)])

    def test_build_require_transitive_shared(self):
        # app -> lib -(br)-> cmake -> zlib1 (shared)
        #          \--(br)-> mingw -> zlib2 (shared) -> SHOULD CONFLICT
        self._cache_recipe("zlib/0.1", GenConanfile().with_shared_option(True))
        self._cache_recipe("zlib/0.2", GenConanfile().with_shared_option(True))
        self._cache_recipe("cmake/0.1", GenConanfile().with_require("zlib/0.1"))
        self._cache_recipe("mingw/0.1", GenConanfile().with_require("zlib/0.2"))
        self._cache_recipe("lib/0.1", GenConanfile().with_tool_requires("cmake/0.1", "mingw/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"),
                                      install=False)

        assert type(deps_graph.error) == GraphConflictError
        out = str(deps_graph.error)
        assert "Version conflict: Conflict between zlib/0.2 and zlib/0.1 in the graph." in out
        assert "Conflict originates from lib/0.1" in out

        assert 6 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst
        mingw = lib.edges[1].dst
        zlib1 = cmake.edges[0].dst
        zlib2 = mingw.edges[0].dst

        assert zlib1 is not zlib2

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake, mingw], dependents=[app])
        self._check_node(cmake, "cmake/0.1#123", deps=[zlib1], dependents=[lib])
        self._check_node(zlib1, "zlib/0.1#123", deps=[], dependents=[cmake])
        self._check_node(mingw, "mingw/0.1#123", deps=[zlib2], dependents=[lib])
        self._check_node(zlib2, "zlib/0.2#123", deps=[], dependents=[mingw])

    def test_build_require_conflict(self):
        # https://github.com/conan-io/conan/issues/4931
        # cheetah -> gazelle -> grass/0.1
        #    \--(br)----------> grass/0.2

        self._cache_recipe("grass/0.1", GenConanfile())
        self._cache_recipe("grass/0.2", GenConanfile())
        self._cache_recipe("gazelle/0.1", GenConanfile().with_require("grass/0.1"))

        deps_graph = self.build_graph(GenConanfile("cheetah", "0.1")
                                      .with_require("gazelle/0.1")
                                      .with_tool_requires("grass/0.2"))

        assert 4 == len(deps_graph.nodes)
        cheetah = deps_graph.root
        gazelle = cheetah.edges[0].dst
        grass2 = cheetah.edges[1].dst
        grass1 = gazelle.edges[0].dst
        self._check_node(cheetah, "cheetah/0.1", deps=[gazelle, grass2])
        self._check_node(gazelle, "gazelle/0.1#123", deps=[grass1], dependents=[cheetah])
        self._check_node(grass1, "grass/0.1#123", deps=[], dependents=[gazelle])
        self._check_node(grass2, "grass/0.2#123", dependents=[cheetah])


class TestBuildRequiresVisible(GraphManagerTest):

    def test_visible_build(self):
        self._cache_recipe("liba/0.1", GenConanfile())
        self._cache_recipe("libb/0.1", GenConanfile().with_requirement("liba/0.1", build=True))
        self._cache_recipe("libc/0.1", GenConanfile().with_requirement("libb/0.1", visible=False))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("libc/0.1"))

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libc = app.edges[0].dst
        libb = libc.edges[0].dst
        liba = libb.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[libc], dependents=[])
        self._check_node(libc, "libc/0.1#123", deps=[libb], dependents=[app])
        self._check_node(libb, "libb/0.1#123", deps=[liba], dependents=[libc])
        self._check_node(liba, "liba/0.1#123", deps=[], dependents=[libb])

        # node, include, link, build, run
        _check_transitive(app, [(libc, True, True, False, False)])
        _check_transitive(libc, [(libb, True, True, False, False),
                                 (liba, False, False, True, False)])  # liba is build & visible!
        _check_transitive(libb, [(liba, True, True, True, False)])


class TestTestRequire(GraphManagerTest):

    def test_basic(self):
        # app -(tr)-> gtest
        self._cache_recipe("gtest/0.1", GenConanfile())
        conanfile = GenConanfile("app", "0.1").with_test_requires("gtest/0.1")

        deps_graph = self.build_graph(conanfile)

        # Build requires always apply to the consumer
        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        gtest = app.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[gtest], dependents=[])
        self._check_node(gtest, "gtest/0.1#123", deps=[], dependents=[app])

        # node, include, link, build, run, test
        _check_transitive(app, [(gtest, True, True, False, False, True)])

    def test_lib_build_require(self):
        # app -> lib -(tr)-> gtest
        self._cache_recipe("gtest/0.1", GenConanfile())
        self._cache_recipe("lib/0.1", GenConanfile().with_test_requires("gtest/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        gtest = lib.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[gtest], dependents=[app])
        self._check_node(gtest, "gtest/0.1#123", deps=[], dependents=[lib])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])
        _check_transitive(lib, [(gtest, True, True, False, False)])

    def test_lib_build_require_transitive(self):
        # app -> lib -(tr)-> gtest
        self._cache_recipe("gtest/0.1", GenConanfile())
        self._cache_recipe("lib/0.1", GenConanfile().with_test_requires("gtest/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        gtest = lib.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[gtest], dependents=[app])
        self._check_node(gtest, "gtest/0.1#123", deps=[], dependents=[lib])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])
        _check_transitive(lib, [(gtest, True, True, False, False)])

    @parameterized.expand([("shared",), ("static",), ("notrun",), ("run",)])
    def test_test_require_transitive(self, gtestlib_type):
        # app -> lib -(tr)-> gtest -> gtestlib (gtestlib_type)

        if gtestlib_type in ("notrun", "run"):  # Unknown
            gtestlib = GenConanfile().with_settings("os")
        else:
            gtestlib = GenConanfile().with_settings("os"). \
                with_shared_option(gtestlib_type == "shared")
        run = True if gtestlib_type == "run" else None  # Not necessary to specify

        self._cache_recipe("gtestlib/0.1", gtestlib)
        self._cache_recipe("gtest/0.1", GenConanfile().with_settings("os").
                           with_requirement("gtestlib/0.1", run=run))
        self._cache_recipe("lib/0.1", GenConanfile().with_settings("os").
                           with_test_requires("gtest/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_settings("os").
                                      with_require("lib/0.1"))

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        gtest = lib.edges[0].dst
        gtestlib = gtest.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[], settings={"os": "Linux"})
        self._check_node(lib, "lib/0.1#123", deps=[gtest], dependents=[app],
                         settings={"os": "Linux"})
        self._check_node(gtest, "gtest/0.1#123", deps=[gtestlib], dependents=[lib],
                         settings={"os": "Linux"})
        self._check_node(gtestlib, "gtestlib/0.1#123", deps=[], dependents=[gtest],
                         settings={"os": "Linux"})

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False)])  # TODO: Check run=None

        if gtestlib_type in ("shared", "run"):
            _check_transitive(lib, [(gtest, True, True, False, False),
                                    (gtestlib, True, True, False, True)])
        elif gtestlib_type == "static":
            _check_transitive(lib, [(gtest, True, True, False, False),
                                    (gtestlib, True, True, False, False)])
        elif gtestlib_type == "notrun":
            _check_transitive(lib, [(gtest, True, True, False, False),
                                    (gtestlib, True, True, False, False)])

    def test_trait_aggregated(self):
        # app -> lib -(tr)-> gtest -> zlib
        #         \-------------------/
        # If zlib is in the host context, a dependency for host, better test=False trait
        self._cache_recipe("zlib/0.1", GenConanfile())
        self._cache_recipe("gtest/0.1", GenConanfile().with_requires("zlib/0.1"))
        self._cache_recipe("lib/0.1", GenConanfile().with_test_requires("gtest/0.1")
                                                    .with_requires("zlib/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_require("lib/0.1"))

        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        gtest = lib.edges[1].dst
        zlib = gtest.edges[0].dst
        zlib2 = lib.edges[0].dst
        assert zlib is zlib2

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[zlib, gtest], dependents=[app])
        self._check_node(gtest, "gtest/0.1#123", deps=[zlib], dependents=[lib])
        self._check_node(zlib, "zlib/0.1#123", deps=[], dependents=[gtest, lib])

        # node, include, link, build, run
        _check_transitive(app, [(lib, True, True, False, False),
                                (zlib, True, True, False, False, False)])
        _check_transitive(lib, [(gtest, True, True, False, False),
                                (zlib, True, True, False, False, False)])

    def test_test_require_loop(self):
        # https://github.com/conan-io/conan/issues/15412
        self._cache_recipe("gtest/1.11", GenConanfile())
        self._cache_recipe("abseil/1.0", GenConanfile().with_test_requires("gtest/[>=1 <1.14]"))
        self._cache_recipe("gtest/1.14", GenConanfile().with_requires("abseil/1.0"))
        deps_graph = self.build_graph(GenConanfile("opencv", "1.0").with_test_requires("gtest/1.14"))

        assert 4 == len(deps_graph.nodes)
        opencv = deps_graph.root
        gtest14 = opencv.edges[0].dst
        abseil = gtest14.edges[0].dst
        gtest11 = abseil.edges[0].dst

        self._check_node(opencv, "opencv/1.0@", deps=[gtest14], dependents=[])
        self._check_node(gtest14, "gtest/1.14#123", deps=[abseil], dependents=[opencv])
        self._check_node(abseil, "abseil/1.0#123", deps=[gtest11], dependents=[gtest14])
        self._check_node(gtest11, "gtest/1.11#123", deps=[], dependents=[abseil])


class TestTestRequiresProblemsShared(GraphManagerTest):

    def _check_graph(self, deps_graph, reverse):
        assert 3 == len(deps_graph.nodes)
        lib_c = deps_graph.root
        if not reverse:
            lib_a = lib_c.edges[0].dst
            util = lib_a.edges[0].dst
            util2 = lib_c.edges[1].dst
        else:
            util = lib_c.edges[0].dst
            lib_a = lib_c.edges[1].dst
            util2 = lib_a.edges[0].dst
        assert util is util2

        self._check_node(lib_c, "lib_c/0.1@", deps=[lib_a, util], dependents=[])
        self._check_node(lib_a, "lib_a/0.1#123", deps=[util], dependents=[lib_c])
        self._check_node(util, "util/0.1#123", deps=[], dependents=[lib_a, lib_c])

        # node, include, link, build, run
        _check_transitive(lib_c, [(lib_a, True, True, False, True),
                                  (util, True, True, False, True)])

    @parameterized.expand([(True,), (False,)])
    def test_fixed_versions(self, reverse):
        #  lib_c -(tr)-> lib_a -0.1--> util
        #    \--------(tr)----0.1------/
        # if versions exactly match, it shouldn't be an issue
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/0.1")
                                                      .with_package_type("shared-library"))
        deps = ("lib_a/0.1", "util/0.1") if not reverse else ("util/0.1", "lib_a/0.1")
        deps_graph = self.build_graph(GenConanfile("lib_c", "0.1").with_test_requires(*deps))
        self._check_graph(deps_graph, reverse)

    @parameterized.expand([(True,), (False,)])
    def test_fixed_versions_conflict(self, reverse):
        #  lib_c -(tr)-> lib_a -0.1--> util
        #    \--------(tr)----0.2------/
        # This should be a a conflict of versions
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("util/0.2", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/0.1")
                                                      .with_package_type("shared-library"))
        deps = ("lib_a/0.1", "util/0.2") if not reverse else ("util/0.2", "lib_a/0.1")
        conanfile = GenConanfile("lib_c", "0.1").with_test_requires(*deps)
        deps_graph = self.build_graph(conanfile, install=False)
        assert type(deps_graph.error) == GraphConflictError

    @parameterized.expand([(True,), (False,)])
    def test_fixed_versions_hybrid(self, reverse):
        #  lib_c -----> lib_a--0.1--> util
        #    \--------(tr)----0.1------/
        # mixing requires + test_requires, should work
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("util/0.2", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/0.1")
                                                      .with_package_type("shared-library"))
        conanfile = GenConanfile("lib_c", "0.1")
        if not reverse:
            conanfile = conanfile.with_requires("lib_a/0.1").with_test_requires("util/0.1")
        else:
            conanfile = conanfile.with_test_requires("lib_a/0.1").with_requires("util/0.1")
        deps_graph = self.build_graph(conanfile)
        self._check_graph(deps_graph, reverse=reverse)

    @parameterized.expand([(True,), (False,)])
    def test_fixed_versions_hybrid_conflict(self, reverse):
        #  lib_c -----> lib_a--0.1---> util
        #    \--------(tr)----0.2------/
        # Same as above, but mixing regular requires with test_requires
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("util/0.2", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/0.1")
                                                      .with_package_type("shared-library"))
        conanfile = GenConanfile("lib_c", "0.1")
        if not reverse:
            conanfile = conanfile.with_requires("lib_a/0.1").with_test_requires("util/0.2")
        else:
            conanfile = conanfile.with_test_requires("lib_a/0.1").with_requires("util/0.2")
        deps_graph = self.build_graph(conanfile, install=False)
        assert type(deps_graph.error) == GraphConflictError

    @parameterized.expand([(True,), (False,)])
    def test_version_ranges(self, reverse):
        #  lib_c -(tr)-> lib_a -> util
        #    \--------(tr)-------/
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/[>=0.1 <1]")
                                                      .with_package_type("shared-library"))

        deps = ("lib_a/[>=0]", "util/[>=0]") if not reverse else ("util/[>=0]", "lib_a/[>=0]")
        deps_graph = self.build_graph(GenConanfile("lib_c", "0.1").with_test_requires(*deps))
        self._check_graph(deps_graph, reverse)

    @parameterized.expand([(True,), (False,)])
    def test_version_ranges_conflict(self, reverse):
        #  lib_c -(tr)-> lib_a -> util/0.1
        #    \--------(tr)------> util/1.0
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("util/1.0", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/[>=0.1 <1]")
                                                      .with_package_type("shared-library"))
        deps = ("lib_a/[>=0]", "util/[>=1]") if not reverse else ("util/[>=1]", "lib_a/[>=0]")
        deps_graph = self.build_graph(GenConanfile("lib_c", "0.1").with_test_requires(*deps),
                                      install=False)
        assert type(deps_graph.error) == GraphConflictError

    @parameterized.expand([(True,), (False,)])
    def test_version_ranges_hybrid(self, reverse):
        #  lib_c ---> lib_a -> util
        #    \--------(tr)-------/
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/[>=0.1 <1]")
                           .with_package_type("shared-library"))

        conanfile = GenConanfile("lib_c", "0.1")
        if not reverse:
            conanfile = conanfile.with_requires("lib_a/[>=0.1]").with_test_requires("util/[>=0.1]")
        else:
            conanfile = conanfile.with_test_requires("lib_a/[>=0.1]").with_requires("util/[>=0.1]")
        deps_graph = self.build_graph(conanfile)
        self._check_graph(deps_graph, reverse)

    @parameterized.expand([(True,), (False,)])
    def test_version_ranges_hybrid_conflict(self, reverse):
        #  lib_c -(tr)-> lib_a -> util/0.1
        #    \--------(tr)------> util/1.0
        self._cache_recipe("util/0.1", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("util/1.0", GenConanfile().with_package_type("shared-library"))
        self._cache_recipe("lib_a/0.1", GenConanfile().with_requires("util/[>=0.1 <1]")
                           .with_package_type("shared-library"))
        conanfile = GenConanfile("lib_c", "0.1")
        if not reverse:
            conanfile = conanfile.with_requires("lib_a/[>=0.1]").with_test_requires("util/[>=1]")
        else:
            conanfile = conanfile.with_test_requires("lib_a/[>=0.1]").with_requires("util/[>=1]")
        deps_graph = self.build_graph(conanfile, install=False)
        assert type(deps_graph.error) == GraphConflictError


class TestBuildRequiresPackageID(GraphManagerTest):

    def test_default_no_affect(self,):
        # app -> lib -(br)-> cmake
        self.recipe_conanfile("cmake/0.1", GenConanfile())
        self.recipe_conanfile("lib/0.1", GenConanfile().with_tool_requires("cmake/0.1"))

        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("lib/0.1"))

        # Build requires always apply to the consumer
        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake], dependents=[app])
        assert lib.package_id == NO_SETTINGS_PACKAGE_ID
        self._check_node(cmake, "cmake/0.1#123", deps=[], dependents=[lib])

    def test_minor_mode(self,):
        # app -> lib -(br)-> cmake
        self.recipe_conanfile("cmake/0.1", GenConanfile())
        self.recipe_conanfile("lib/0.1", GenConanfile().
                              with_tool_requirement("cmake/[*]", package_id_mode="minor_mode"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("lib/0.1"))

        # Build requires always apply to the consumer
        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake], dependents=[app])
        assert lib.package_id == "6b92478cba14dbdc06d4e991430d3c1c04959d4a"
        assert lib.package_id != NO_SETTINGS_PACKAGE_ID
        self._check_node(cmake, "cmake/0.1#123", deps=[], dependents=[lib])

        # Change the dependency to next minor
        self.recipe_conanfile("cmake/0.2", GenConanfile())
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("lib/0.1"))

        # Build requires always apply to the consumer
        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        assert lib.package_id == "2813db72897dd13aca2af071efe8ecb116f679ed"
        assert lib.package_id != NO_SETTINGS_PACKAGE_ID


class TestPublicBuildRequires(GraphManagerTest):

    def test_simple(self):
        # app -> lib -(br public)-> cmake
        self.recipe_conanfile("cmake/0.1", GenConanfile())
        self.recipe_conanfile("lib/0.1", GenConanfile()
                              .with_tool_requirement("cmake/0.1", visible=True))

        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("lib/0.1"))

        # Build requires always apply to the consumer
        assert 3 == len(deps_graph.nodes)
        app = deps_graph.root
        lib = app.edges[0].dst
        cmake = lib.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[lib], dependents=[])
        self._check_node(lib, "lib/0.1#123", deps=[cmake], dependents=[app])
        self._check_node(cmake, "cmake/0.1#123", deps=[], dependents=[lib])

        # node, include, link, build, run
        _check_transitive(lib, [(cmake, False, False, True, True)])
        _check_transitive(app, [(lib, True, True, False, False),
                                (cmake, False, False, True, True)])

    def test_deep_dependency_tree(self):
        # app -> liba -> libb-(br public) -> sfun -> libsfun -> libx -> liby -> libz
        #                    -(normal req) -> libsfun -> libx -> liby -> libz
        self.recipe_conanfile("libz/0.1", GenConanfile())
        self.recipe_conanfile("liby/0.1", GenConanfile().with_requirement("libz/0.1", run=True))
        self.recipe_conanfile("libx/0.1", GenConanfile().with_requirement("liby/0.1", run=True))
        self.recipe_conanfile("libsfun/0.1", GenConanfile().with_requirement("libx/0.1", run=True))
        self.recipe_conanfile("sfun/0.1", GenConanfile().with_requirement("libsfun/0.1", run=True))
        self.recipe_conanfile("libb/0.1", GenConanfile()
                              .with_tool_requirement("sfun/0.1", visible=True)
                              .with_requirement("libsfun/0.1", run=True))
        self.recipe_conanfile("liba/0.1", GenConanfile().with_requirement("libb/0.1", run=True))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requirement("liba/0.1", run=True))

        # Build requires always apply to the consumer
        assert 8 + 4 == len(deps_graph.nodes)
        app = deps_graph.root
        liba = app.edges[0].dst
        libb = liba.edges[0].dst
        libsfun = libb.edges[0].dst
        libx = libsfun.edges[0].dst
        liby = libx.edges[0].dst
        libz = liby.edges[0].dst
        sfun = libb.edges[1].dst
        libsfun_build = sfun.edges[0].dst
        libx_build = libsfun_build.edges[0].dst
        liby_build = libx_build.edges[0].dst
        libz_build = liby_build.edges[0].dst

        # TODO non-build-requires

        self._check_node(app, "app/0.1@", deps=[liba], dependents=[])
        self._check_node(liba, "liba/0.1#123", deps=[libb], dependents=[app])
        self._check_node(libb, "libb/0.1#123", deps=[sfun, libsfun], dependents=[liba])
        self._check_node(sfun, "sfun/0.1#123", deps=[libsfun_build], dependents=[libb])
        self._check_node(libsfun_build, "libsfun/0.1#123", deps=[libx_build], dependents=[sfun])
        self._check_node(libx_build, "libx/0.1#123", deps=[liby_build], dependents=[libsfun_build])
        self._check_node(liby_build, "liby/0.1#123", deps=[libz_build], dependents=[libx_build])
        self._check_node(libz_build, "libz/0.1#123", deps=[], dependents=[liby_build])

        # node, include, link, build, run
        _check_transitive(liby_build, [(libz_build, True, True, False, True)])
        _check_transitive(libx_build, [(liby_build, True, True, False, True),
                                       (libz_build, True, True, False, True)])
        _check_transitive(libsfun_build, [(libx_build, True, True, False, True),
                                          (liby_build, True, True, False, True),
                                          (libz_build, True, True, False, True)])
        _check_transitive(sfun, [(libsfun_build, True, True, False, True),
                                 (libx_build, True, True, False, True),
                                 (liby_build, True, True, False, True),
                                 (libz_build, True, True, False, True)])
        _check_transitive(libb, [(libsfun, True, True, False, True),
                                 (libx, True, True, False, True),
                                 (liby, True, True, False, True),
                                 (libz, True, True, False, True),
                                 (sfun, False, False, True, True),
                                 (libsfun_build, False, False, True, True),
                                 (libx_build, False, False, True, True),
                                 (liby_build, False, False, True, True),
                                 (libz_build, False, False, True, True)])
        _check_transitive(liba, [(libb, True, True, False, True),
                                 (libsfun, True, True, False, True),
                                 (libx, True, True, False, True),
                                 (liby, True, True, False, True),
                                 (libz, True, True, False, True),
                                 (sfun, False, False, True, True),
                                 (libsfun_build, False, False, True, True),
                                 (libx_build, False, False, True, True),
                                 (liby_build, False, False, True, True),
                                 (libz_build, False, False, True, True)])
        _check_transitive(app, [(liba, True, True, False, True),
                                (libb, True, True, False, True),
                                (libsfun, True, True, False, True),
                                (libx, True, True, False, True),
                                (liby, True, True, False, True),
                                (libz, True, True, False, True),
                                (sfun, False, False, True, True),
                                (libsfun_build, False, False, True, True),
                                (libx_build, False, False, True, True),
                                (liby_build, False, False, True, True),
                                (libz_build, False, False, True, True)])

    def test_conflict_diamond(self):
        # app -> libb -(br public)-> cmake/0.1
        #   \--> libc -(br public)-> cmake/0.2
        self.recipe_conanfile("cmake/0.1", GenConanfile())
        self.recipe_conanfile("cmake/0.2", GenConanfile())
        self.recipe_conanfile("libb/0.1",
                              GenConanfile().with_tool_requirement("cmake/0.1", visible=True))
        self.recipe_conanfile("libc/0.1",
                              GenConanfile().with_tool_requirement("cmake/0.2", visible=True))

        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("libb/0.1",
                                                                               "libc/0.1"),
                                      install=False)

        assert type(deps_graph.error) == GraphConflictError

        # Build requires always apply to the consumer
        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        libc = app.edges[1].dst
        cmake1 = libb.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[libb, libc], dependents=[])
        self._check_node(libb, "libb/0.1#123", deps=[cmake1], dependents=[app])
        self._check_node(cmake1, "cmake/0.1#123", deps=[], dependents=[libb])

    def test_conflict_diamond_two_levels(self):
        # app -> libd -> libb -(br public)-> cmake/0.1
        #   \--> libe -> libc -(br public)-> cmake/0.2
        self.recipe_conanfile("cmake/0.1", GenConanfile())
        self.recipe_conanfile("cmake/0.2", GenConanfile())
        self.recipe_conanfile("libb/0.1",
                              GenConanfile().with_tool_requirement("cmake/0.1", visible=True))
        self.recipe_conanfile("libc/0.1",
                              GenConanfile().with_tool_requirement("cmake/0.2", visible=True))
        self.recipe_conanfile("libd/0.1", GenConanfile().with_requires("libb/0.1"))
        self.recipe_conanfile("libe/0.1", GenConanfile().with_requires("libc/0.1"))

        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("libd/0.1",
                                                                               "libe/0.1"),
                                      install=False)

        assert type(deps_graph.error) == GraphConflictError

        # Build requires always apply to the consumer
        assert 6 == len(deps_graph.nodes)
        app = deps_graph.root
        libd = app.edges[0].dst
        libe = app.edges[1].dst
        libb = libd.edges[0].dst
        libc = libe.edges[0].dst
        cmake1 = libb.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[libd, libe], dependents=[])
        self._check_node(libd, "libd/0.1#123", deps=[libb], dependents=[app])
        self._check_node(libe, "libe/0.1#123", deps=[libc], dependents=[app])
        self._check_node(libb, "libb/0.1#123", deps=[cmake1], dependents=[libd])
        self._check_node(libc, "libc/0.1#123", deps=[], dependents=[libe])
        self._check_node(cmake1, "cmake/0.1#123", deps=[], dependents=[libb])

    def test_tool_requires(self):
        # app -> libb -(br public)-> protobuf/0.1
        #           \--------------> protobuf/0.2
        self.recipe_conanfile("protobuf/0.1", GenConanfile())
        self.recipe_conanfile("protobuf/0.2", GenConanfile())
        self.recipe_conanfile("libb/0.1",
                              GenConanfile().with_tool_requirement("protobuf/0.1", visible=True)
                              .with_require("protobuf/0.2"))

        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("libb/0.1"))

        # Build requires always apply to the consumer
        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        protobuf_host = libb.edges[0].dst
        protobuf_build = libb.edges[1].dst

        self._check_node(app, "app/0.1@", deps=[libb], dependents=[])
        self._check_node(libb, "libb/0.1#123", deps=[protobuf_host, protobuf_build],
                         dependents=[app])
        self._check_node(protobuf_host, "protobuf/0.2#123", deps=[], dependents=[libb])
        self._check_node(protobuf_build, "protobuf/0.1#123", deps=[], dependents=[libb])

        # node, headers, lib, build, run
        _check_transitive(app, [(libb, True, True, False, False),
                                (protobuf_host, True, True, False, False),
                                (protobuf_build, False, False, True, True)])

    def test_tool_requires_override(self):
        # app -> libb -(br public)-> protobuf/0.1
        #           \--------------> protobuf/0.2
        #  \---(br, override)------> protobuf/0.2
        self.recipe_conanfile("protobuf/0.1", GenConanfile())
        self.recipe_conanfile("protobuf/0.2", GenConanfile())
        self.recipe_conanfile("libb/0.1",
                              GenConanfile().with_tool_requirement("protobuf/0.1", visible=True)
                              .with_require("protobuf/0.2"))

        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_requires("libb/0.1")
                                      .with_tool_requirement("protobuf/0.2", override=True))

        # Build requires always apply to the consumer
        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        libb = app.edges[0].dst
        protobuf_host = libb.edges[0].dst
        protobuf_build = libb.edges[1].dst

        self._check_node(app, "app/0.1@", deps=[libb], dependents=[])
        self._check_node(libb, "libb/0.1#123", deps=[protobuf_host, protobuf_build],
                         dependents=[app])
        self._check_node(protobuf_host, "protobuf/0.2#123", deps=[], dependents=[libb])
        self._check_node(protobuf_build, "protobuf/0.2#123", deps=[], dependents=[libb])

        # node, headers, lib, build, run
        _check_transitive(app, [(libb, True, True, False, False),
                                (protobuf_host, True, True, False, False),
                                (protobuf_build, False, False, True, True)])
        _check_transitive(libb, [(protobuf_host, True, True, False, False),
                                 (protobuf_build, False, False, True, True)])

    def test_test_require(self):
        # app -(tr)-> gtest/0.1
        # This test should survive in 2.0
        tool_ref = RecipeReference.loads("gtest/0.1")
        self._cache_recipe(tool_ref, GenConanfile("gtest", "0.1"))

        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "app"
                version = "0.1"
                def build_requirements(self):
                    self.test_requires("gtest/0.1")
            """)
        deps_graph = self.build_graph(conanfile)

        # Build requires always apply to the consumer
        assert 2 == len(deps_graph.nodes)
        app = deps_graph.root
        tool = app.edges[0].dst

        self._check_node(app, "app/0.1@", deps=[tool], dependents=[])
        self._check_node(tool, "gtest/0.1#123", deps=[], dependents=[app])


class TestLoops(GraphManagerTest):

    def test_direct_loop_error(self):
        # app -(br)-> cmake/0.1 -(br itself)-> cmake/0.1....
        # causing an infinite loop
        self._cache_recipe("cmake/0.1", GenConanfile().with_tool_requires("cmake/0.1"))
        deps_graph = self.build_graph(GenConanfile("app", "0.1").with_build_requires("cmake/0.1"),
                                      install=False)

        assert type(deps_graph.error) == GraphLoopError

        # Build requires always apply to the consumer
        assert 4 == len(deps_graph.nodes)
        app = deps_graph.root
        tool = app.edges[0].dst
        tool2 = tool.edges[0].dst
        tool3 = tool2.edges[0].dst

        self._check_node(app, "app/0.1", deps=[tool], dependents=[])
        self._check_node(tool, "cmake/0.1#123", deps=[tool2], dependents=[app])
        self._check_node(tool2, "cmake/0.1#123", deps=[tool3], dependents=[tool])

    def test_indirect_loop_error(self):
        # app -(br)-> gtest/0.1 -(br)-> cmake/0.1 -(br)->gtest/0.1 ....
        # causing an infinite loop
        self._cache_recipe("gtest/0.1", GenConanfile().with_tool_requires("cmake/0.1"))
        self._cache_recipe("cmake/0.1", GenConanfile().with_test_requires("gtest/0.1"))

        deps_graph = self.build_graph(GenConanfile().with_build_requires("cmake/0.1"),
                                      install=False)

        assert type(deps_graph.error) == GraphLoopError

        # Build requires always apply to the consumer
        assert 6 == len(deps_graph.nodes)
        app = deps_graph.root
        cmake = app.edges[0].dst
        gtest = cmake.edges[0].dst
        cmake2 = gtest.edges[0].dst
        gtest2 = cmake2.edges[0].dst
        cmake3 = gtest2.edges[0].dst

        assert deps_graph.error.ancestor == cmake
        assert deps_graph.error.node == cmake3

    def test_infinite_recursion_test(self):
        """Ensure that direct conflicts in a node are properly reported and Conan does not loop"""
        tc = TestClient(light=True)
        tc.save({"conanfile.py": GenConanfile("app").with_package_type("application")})
        tc.run("create . --version=1.0 --build-require")
        tc.run("create . --version=1.1 --build-require")
        tc.run("graph info --tool-requires=app/1.1 --tool-requires=app/1.0", assert_error=True)
        assert "Duplicated requirement: app/1.0" in tc.out


def test_tool_requires():
    """Testing temporary tool_requires attribute being "an alias" of build_require and
    introduced to provide a compatible recipe with 2.0. At 2.0, the meaning of a build require being
    a 'tool' will be a tool_require."""

    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile()})
    client.run("create . --name=tool1 --version=1.0")
    client.run("create . --name=tool2 --version=1.0")
    client.run("create . --name=tool3 --version=1.0")
    client.run("create . --name=tool4 --version=1.0")

    consumer = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            tool_requires = "tool2/1.0"
            build_requires = "tool3/1.0"

            def build_requirements(self):
                self.tool_requires("tool1/1.0")
                self.build_requires("tool4/1.0")

            def generate(self):
                assert len(self.dependencies.build.values()) == 4
    """)
    client.save({"conanfile.py": consumer})
    client.run("create . --name=consumer --version=1.0")
    client.assert_listed_require({"tool1/1.0": "Cache",
                                  "tool2/1.0": "Cache",
                                  "tool3/1.0": "Cache",
                                  "tool4/1.0": "Cache"}, build=True)


class TestDuplicateBuildRequires:
    """ what happens when you require and tool_require the same dependency
    """
    # https://github.com/conan-io/conan/issues/11179

    @pytest.fixture()
    def client(self):
        client = TestClient()
        msg = "self.output.info('This is the binary for OS={}'.format(self.info.settings.os))"
        msg2 = "self.output.info('This is in context={}'.format(self.context))"
        client.save({"conanfile.py": GenConanfile().with_settings("os").with_package_id(msg)
                                                                       .with_package_id(msg2)})
        client.run("create . --name=tool1 --version=1.0 -s os=Windows")
        client.run("create . --name=tool2 --version=1.0 -s os=Windows")
        client.run("create . --name=tool3 --version=1.0 -s os=Windows")
        client.run("create . --name=tool4 --version=1.0 -s os=Windows")

        consumer = textwrap.dedent("""
            from conan import ConanFile
            class Pkg(ConanFile):
                name = "consumer"
                version = "1.0"
                tool_requires = "tool2/1.0"
                build_requires = "tool3/1.0"
                def requirements(self):
                    self.requires("tool4/1.0")
                def build_requirements(self):
                    self.build_requires("tool4/1.0")
                    self.tool_requires("tool1/1.0")
                def generate(self):
                    host_deps = sorted([d.ref for d, _ in self.dependencies.host.items()])
                    build_deps = sorted([d.ref for d, _ in self.dependencies.build.items()])
                    self.output.info("HOST DEPS: {}".format(host_deps))
                    self.output.info("BUILD DEPS: {}".format(build_deps))
                    assert len(host_deps) == 1, host_deps
                    assert len(build_deps) == 4, build_deps
            """)
        client.save({"conanfile.py": consumer})
        return client

    def test_tool_requires_in_test_package(self, client):
        """Test that tool requires can be listed as build and host requirements"""
        test = textwrap.dedent("""\
            from conan import ConanFile
            class Test(ConanFile):
                def build_requirements(self):
                    self.tool_requires(self.tested_reference_str)
                def test(self):
                    pass
        """)

        client.save({"test_package/conanfile.py": test})
        client.run("create . -s:b os=Windows -s:h os=Linux  --build-require")
        assert "This is the binary for OS=Linux" not in client.out
        assert "This is in context=host" not in client.out
        client.assert_listed_require({"consumer/1.0": "Cache"}, build=True)
        for tool in "tool1", "tool2", "tool3", "tool4":
            client.assert_listed_require({f"{tool}/1.0": "Cache"}, build=True)
            assert f"{tool}/1.0: This is the binary for OS=Windows" in client.out
            assert f"{tool}/1.0: This is in context=build" in client.out

        assert "consumer/1.0: HOST DEPS: [tool4/1.0]" in client.out
        assert "consumer/1.0: BUILD DEPS: [tool1/1.0, tool2/1.0, tool3/1.0, tool4/1.0]" in client.out

    def test_test_requires_in_test_package(self, client):
        """Test that tool requires can be listed as build and host requirements"""
        test = textwrap.dedent("""\
            from conan import ConanFile
            class Test(ConanFile):
                def build_requirements(self):
                    self.test_requires(self.tested_reference_str)
                def test(self):
                    pass
        """)

        client.save({"test_package/conanfile.py": test})
        client.run("create . -s:b os=Windows -s:h os=Linux --build=missing")
        for tool in "tool1", "tool2", "tool3", "tool4":
            client.assert_listed_require({f"{tool}/1.0": "Cache"}, build=True)
            assert f"{tool}/1.0: This is the binary for OS=Windows" in client.out
            assert f"{tool}/1.0: This is in context=build" in client.out
        client.assert_listed_require({"consumer/1.0": "Cache",
                                      "tool4/1.0": "Cache"})
        client.assert_listed_binary({"tool4/1.0": ("9a4eb3c8701508aa9458b1a73d0633783ecc2270",
                                                   "Build")})

        assert "tool4/1.0: This is the binary for OS=Linux" in client.out
        assert "tool4/1.0: This is in context=host" in client.out

        assert "consumer/1.0: HOST DEPS: [tool4/1.0]" in client.out
        assert "consumer/1.0: BUILD DEPS: [tool1/1.0, tool2/1.0, tool3/1.0, tool4/1.0]" in client.out



import os.path
import textwrap
from collections import OrderedDict

import pytest

from conan.test.utils.tools import TestClient, TestServer


class TestMetadataDeploy:
    """ prove we can gather metadata too with a deployer"""

    @pytest.fixture
    def client(self):
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import save, copy

            class Pkg(ConanFile):
                version = "0.1"

                def source(self):
                    save(self, os.path.join(self.recipe_metadata_folder, "logs", "src.log"),
                         f"srclog {self.name}!!!")

                def build(self):
                    save(self, "mylogs.txt", f"some logs {self.name}!!!")
                    copy(self, "mylogs.txt", src=self.build_folder,
                         dst=os.path.join(self.package_metadata_folder, "logs"))
            """)
        deploy = textwrap.dedent("""
            import os, shutil

            def deploy(graph, output_folder, **kwargs):
                conanfile = graph.root.conanfile
                for r, d in conanfile.dependencies.items():
                    if not os.listdir(d.package_metadata_folder):
                        continue
                    shutil.copytree(d.package_metadata_folder, os.path.join(output_folder, "pkgs",
                                                                            d.ref.name))
                    shutil.copytree(d.recipe_metadata_folder, os.path.join(output_folder, "recipes",
                                                                             d.ref.name))
           """)

        servers = OrderedDict([("default", TestServer()), ("remote2", TestServer())])
        c = TestClient(servers=servers, inputs=2 * ["admin", "password"], light=True)
        c.save({"conanfile.py": conanfile,
                "deploy.py": deploy})
        c.run("create . --name=pkg1")
        c.run("create . --name=pkg2")
        return c

    def test_cache(self, client):
        c = client
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1 --deployer=deploy")
        assert "some logs pkg1!!!" in c.load("pkgs/pkg1/logs/mylogs.txt")
        assert "some logs pkg2!!!" in c.load("pkgs/pkg2/logs/mylogs.txt")
        assert "srclog pkg1!!!" in c.load("recipes/pkg1/logs/src.log")
        assert "srclog pkg2!!!" in c.load("recipes/pkg2/logs/src.log")

    def test_remote(self, client):
        # But the remote story is more complex, metadata is not retrieved by default
        c = client
        c.run("upload * -c -r=default")
        c.run("remove * -c")
        # First install without metadata
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1")
        # So this will not deploy metadata
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1 --deployer=deploy -f=json",
              redirect_stdout="graph.json")
        assert not os.path.exists(os.path.join(c.current_folder, "pkgs"))

        # We can obtain the pkg-list for the graph, then "find-remote" and download the metadata
        c.run("list -g=graph.json -f=json", redirect_stdout="mylist.json")
        c.run("pkglist find-remote mylist.json -f=json", redirect_stdout="pkg_remotes.json")
        c.run("download --list=pkg_remotes.json -r=default --metadata=*")

        # Now we will have the metadata in cache and we can deploy it
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1 --deployer=deploy")
        assert "some logs pkg1!!!" in c.load("pkgs/pkg1/logs/mylogs.txt")
        assert "some logs pkg2!!!" in c.load("pkgs/pkg2/logs/mylogs.txt")
        assert "srclog pkg1!!!" in c.load("recipes/pkg1/logs/src.log")
        assert "srclog pkg2!!!" in c.load("recipes/pkg2/logs/src.log")

    def test_multi_remote(self, client):
        # But the remote story is more complex, metadata is not retrieved by default,
        # we need to iterate the remotes if data coming from multiple remotes
        c = client
        c.run("upload pkg1* -c -r=default")
        c.run("upload pkg2* -c -r=remote2")
        c.run("remove * -c")
        # First install without metadata
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1")
        # So this will not deploy metadata
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1 --deployer=deploy -f=json",
              redirect_stdout="graph.json")
        assert not os.path.exists(os.path.join(c.current_folder, "pkgs"))

        # We can obtain the pkg-list for the graph, then "find-remote" and download the metadata
        c.run("list -g=graph.json -f=json", redirect_stdout="mylist.json")
        c.run("pkglist find-remote mylist.json -f=json", redirect_stdout="pkg_remotes.json")
        # we need to ITERATE the remotes
        c.run("download --list=pkg_remotes.json -r=default --metadata=*")
        c.run("download --list=pkg_remotes.json -r=remote2 --metadata=*")

        # Now we will have the metadata in cache and we can deploy it
        c.run("install --requires=pkg1/0.1 --requires=pkg2/0.1 --deployer=deploy")
        assert "some logs pkg1!!!" in c.load("pkgs/pkg1/logs/mylogs.txt")
        assert "some logs pkg2!!!" in c.load("pkgs/pkg2/logs/mylogs.txt")
        assert "srclog pkg1!!!" in c.load("recipes/pkg1/logs/src.log")
        assert "srclog pkg2!!!" in c.load("recipes/pkg2/logs/src.log")



import os

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.tool("cmake")
def test_cmake_lib_template():
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_lib -d name=hello -d version=0.1")
    # Local flow works
    client.run("build .")

    client.run("export-pkg .")
    package_folder = client.created_layout().package()
    assert os.path.exists(os.path.join(package_folder, "include", "hello.h"))


@pytest.mark.tool("cmake")
def test_cmake_lib_template_create(matrix_client_shared_debug):
    client = matrix_client_shared_debug
    client.run("new cmake_lib -d name=matrix -d version=1.0")
    # Create works
    client.run("create . --build=never")
    assert "matrix/1.0: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug --build=never")
    assert "matrix/1.0: Hello World Debug!" in client.out

    # Create + shared works
    client.run("create . -o hello/*:shared=True --build=never")
    assert "matrix/1.0: Hello World Release!" in client.out


@pytest.mark.tool("cmake")
def test_cmake_exe_template():
    client = TestClient(path_with_spaces=False)
    client.run("new cmake_exe -d name=greet -d version=0.1")
    # Local flow works
    client.run("build .")

    # Create works
    client.run("create .")
    assert "greet/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "greet/0.1: Hello World Debug!" in client.out



import pytest

from conan.tools.gnu.get_gnu_triplet import _get_gnu_triplet, _get_gnu_os, _get_gnu_arch
from conan.errors import ConanException


@pytest.mark.parametrize("os_, arch, compiler, expected", [
    ["Linux", "x86", None, "i686-linux-gnu"],
    ["Linux", "x86_64", None, "x86_64-linux-gnu"],
    ["Linux", "armv6", None, "arm-linux-gnueabi"],
    ["Linux", "sparc", None, "sparc-linux-gnu"],
    ["Linux", "sparcv9", None, "sparc64-linux-gnu"],
    ["Linux", "mips", None, "mips-linux-gnu"],
    ["Linux", "mips64", None, "mips64-linux-gnu"],
    ["Linux", "ppc32", None, "powerpc-linux-gnu"],
    ["Linux", "ppc64", None, "powerpc64-linux-gnu"],
    ["Linux", "ppc64le", None, "powerpc64le-linux-gnu"],
    ["Linux", "armv5te", None, "arm-linux-gnueabi"],
    ["Linux", "arm_whatever", None, "arm-linux-gnueabi"],
    ["Linux", "armv7hf", None, "arm-linux-gnueabihf"],
    ["Linux", "armv6", None, "arm-linux-gnueabi"],
    ["Linux", "armv7", None, "arm-linux-gnueabi"],
    ["Linux", "armv8_32", None, "aarch64-linux-gnu_ilp32"],
    ["Linux", "armv5el", None, "arm-linux-gnueabi"],
    ["Linux", "armv5hf", None, "arm-linux-gnueabihf"],
    ["Linux", "s390", None, "s390-ibm-linux-gnu"],
    ["Linux", "s390x", None, "s390x-ibm-linux-gnu"],
    ["Android", "x86", None, "i686-linux-android"],
    ["Android", "x86_64", None, "x86_64-linux-android"],
    ["Android", "armv6", None, "arm-linux-androideabi"],
    ["Android", "armv7", None, "arm-linux-androideabi"],
    ["Android", "armv7hf", None, "arm-linux-androideabi"],
    ["Android", "armv8", None, "aarch64-linux-android"],
    ["Windows", "x86", "msvc", "i686-unknown-windows"],
    ["Windows", "x86_64", "msvc", "x86_64-unknown-windows"],
    ["Windows", "armv8", "msvc", "aarch64-unknown-windows"],
    ["Windows", "x86", "gcc", "i686-w64-mingw32"],
    ["Windows", "x86_64", "gcc", "x86_64-w64-mingw32"],
    ["Darwin", "x86_64", None, "x86_64-apple-darwin"],
    ["Macos", "x86", None, "i686-apple-darwin"],
    ["iOS", "armv7", None, "arm-apple-ios"],
    ["iOS", "x86", None, "i686-apple-ios"],
    ["iOS", "x86_64", None, "x86_64-apple-ios"],
    ["watchOS", "armv7k", None, "arm-apple-watchos"],
    ["watchOS", "armv8_32", None, "aarch64-apple-watchos"],
    ["watchOS", "x86", None, "i686-apple-watchos"],
    ["watchOS", "x86_64", None, "x86_64-apple-watchos"],
    ["tvOS", "armv8", None, "aarch64-apple-tvos"],
    ["tvOS", "armv8.3", None, "aarch64-apple-tvos"],
    ["tvOS", "x86", None, "i686-apple-tvos"],
    ["tvOS", "x86_64", None, "x86_64-apple-tvos"],
    ["Emscripten", "asm.js", None, "asmjs-local-emscripten"],
    ["Emscripten", "wasm", None, "wasm32-local-emscripten"],
    ["Emscripten", "wasm64", None, "wasm64-local-emscripten"],
    ["AIX", "ppc32", None, "rs6000-ibm-aix"],
    ["AIX", "ppc64", None, "powerpc-ibm-aix"],
    ["Neutrino", "armv7", None, "arm-nto-qnx"],
    ["Neutrino", "armv8", None, "aarch64-nto-qnx"],
    ["Neutrino", "sh4le", None, "sh4-nto-qnx"],
    ["Neutrino", "ppc32be", None, "powerpcbe-nto-qnx"],
    ["Linux", "e2k-v2", None, "e2k-unknown-linux-gnu"],
    ["Linux", "e2k-v3", None, "e2k-unknown-linux-gnu"],
    ["Linux", "e2k-v4", None, "e2k-unknown-linux-gnu"],
    ["Linux", "e2k-v5", None, "e2k-unknown-linux-gnu"],
    ["Linux", "e2k-v6", None, "e2k-unknown-linux-gnu"],
    ["Linux", "e2k-v7", None, "e2k-unknown-linux-gnu"],
    ["Linux", "riscv32", None, "riscv32-linux-gnu"],
    ["Linux", "riscv64", None, "riscv64-linux-gnu"],
])
def test_get_gnu_triplet(os_, arch, compiler, expected):
    info = _get_gnu_triplet(os_, arch, compiler)
    assert info["triplet"] == expected
    assert info["machine"] == _get_gnu_arch(os_, arch)
    assert info["system"] == _get_gnu_os(os_, arch, compiler)


def test_get_gnu_triplet_on_windows_without_compiler():
    with pytest.raises(ConanException):
        _get_gnu_triplet("Windows", "x86")



import os
import textwrap

import pytest

from conan.api.model import RecipeReference
from conan.test.utils.tools import TestClient
from conan.internal.util.files import load, save


class TestRecipeMetadataLogs:

    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save, copy

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def export(self):
                copy(self, "*.log", src=self.recipe_folder,
                dst=os.path.join(self.recipe_metadata_folder, "logs"))

            def layout(self):
                self.folders.build = "mybuild"
                self.folders.generators = "mybuild/generators"

            def source(self):
                save(self, os.path.join(self.recipe_metadata_folder, "logs", "src.log"), "srclog!!")

            def build(self):
                save(self, "mylogs.txt", "some logs!!!")
                copy(self, "mylogs.txt", src=self.build_folder,
                dst=os.path.join(self.package_metadata_folder, "logs"))
        """)

    def test_metadata_logs(self):
        c = TestClient(default_server_user=True)
        c.save({"conanfile.py": self.conanfile,
                "file.log": "log contents!"})
        c.run("create .")
        # Test local cache looks good
        ref = RecipeReference.loads("pkg/0.1")
        ref_layout = c.get_latest_ref_layout(ref)
        assert os.listdir(ref_layout.metadata()) == ["logs"]
        assert set(os.listdir(os.path.join(ref_layout.metadata(), "logs"))) == {"file.log",
                                                                                "src.log"}
        assert load(os.path.join(ref_layout.metadata(), "logs", "file.log")) == "log contents!"
        assert load(os.path.join(ref_layout.metadata(), "logs", "src.log")) == "srclog!!"

        pref = c.get_latest_package_reference(ref)
        pref_layout = c.get_latest_pkg_layout(pref)
        assert os.listdir(pref_layout.metadata()) == ["logs"]
        assert os.listdir(os.path.join(pref_layout.metadata(), "logs")) == ["mylogs.txt"]
        assert load(os.path.join(pref_layout.metadata(), "logs", "mylogs.txt")) == "some logs!!!"

    def test_metadata_logs_local(self):
        c = TestClient(default_server_user=True)
        c.save({"conanfile.py": self.conanfile,
                "file.log": "log contents!"})
        c.run("source .")
        assert c.load("metadata/logs/src.log") == "srclog!!"
        c.run("build .")
        assert c.load("mybuild/metadata/logs/mylogs.txt") == "some logs!!!"

    def test_download_pkg_list_from_graph(self):
        c = TestClient(default_server_user=True)
        c.save({"conanfile.py": self.conanfile,
                "file.log": "log contents!"})
        c.run("create .")
        c.run("upload * -r=default -c")
        c.run("remove * -c")
        #  IMPORTANT: NECESSARY to force the download to gather the full package_list
        # TODO: Check the case how to download metadata for already installed in cache packages
        c.run("install --requires=pkg/0.1 --format=json", redirect_stdout="graph.json")
        c.run("list --graph=graph.json --format=json", redirect_stdout="pkglist.json")
        # This list will contain both "Local Cache" and "default" origins, because it was downloaded
        c.run("download --list=pkglist.json -r=default --metadata=*")

        ref = RecipeReference.loads("pkg/0.1")
        pref = c.get_latest_package_reference(ref)
        pref_layout = c.get_latest_pkg_layout(pref)
        assert os.listdir(pref_layout.metadata()) == ["logs"]
        assert os.listdir(os.path.join(pref_layout.metadata(), "logs")) == ["mylogs.txt"]
        assert load(os.path.join(pref_layout.metadata(), "logs", "mylogs.txt")) == "some logs!!!"

    def test_metadata_folder_exist(self):
        """ make sure the folders exists
        so recipe don't have to create it for running bulk copies calling self.run(cp -R)
        """
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile

            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"

                def export(self):
                    assert os.path.exists(self.recipe_metadata_folder)

                def source(self):
                    assert os.path.exists(self.recipe_metadata_folder)

                def build(self):
                    assert os.path.exists(self.package_metadata_folder)
            """)
        c = TestClient(default_server_user=True)
        c.save({"conanfile.py": conanfile})
        c.run("create .")
        c.run("upload * -r=default -c")
        c.run("remove * -c")
        c.run("install --requires=pkg/0.1 --build=*")
        # If nothing fail, all good, all folder existed, assert passed


class TestHooksMetadataLogs:

    @pytest.fixture()
    def _client(self):
        c = TestClient(default_server_user=True)
        my_hook = textwrap.dedent("""\
            import os
            from conan.tools.files import copy

            def post_export(conanfile):
                conanfile.output.info("post_export")
                copy(conanfile, "*.log", src=conanfile.recipe_folder,
                     dst=os.path.join(conanfile.recipe_metadata_folder, "logs"))

            def post_source(conanfile):
                conanfile.output.info("post_source")
                copy(conanfile, "*", src=os.path.join(conanfile.source_folder, "logs"),
                     dst=os.path.join(conanfile.recipe_metadata_folder, "logs"))

            def post_build(conanfile):
                conanfile.output.info("post_build")
                copy(conanfile, "*", src=os.path.join(conanfile.build_folder, "logs"),
                     dst=os.path.join(conanfile.package_metadata_folder, "logs"))
            """)
        hook_path = os.path.join(c.paths.hooks_path, "my_hook", "hook_my_hook.py")
        save(hook_path, my_hook)
        conanfile = textwrap.dedent("""
            import os
            from conan import ConanFile
            from conan.tools.files import save, copy

            class Pkg(ConanFile):
                name = "pkg"
                version = "0.1"
                no_copy_source = True

                def layout(self):
                    self.folders.build = "mybuild"
                    self.folders.generators = "mybuild/generators"

                def source(self):
                    save(self, "logs/src.log", "srclog!!")

                def build(self):
                    save(self, "logs/mylogs.txt", "some logs!!!")
            """)
        c.save({"conanfile.py": conanfile,
                "file.log": "log contents!"})
        return c

    def test_metadata_logs_hook(self, _client):
        c = _client
        c.run("create .")
        # Test local cache looks good
        ref = RecipeReference.loads("pkg/0.1")
        ref_layout = c.get_latest_ref_layout(ref)
        assert os.listdir(ref_layout.metadata()) == ["logs"]
        assert set(os.listdir(os.path.join(ref_layout.metadata(), "logs"))) == {"file.log",
                                                                                "src.log"}
        assert load(os.path.join(ref_layout.metadata(), "logs", "file.log")) == "log contents!"
        assert load(os.path.join(ref_layout.metadata(), "logs", "src.log")) == "srclog!!"

        pref = c.get_latest_package_reference(ref)
        pref_layout = c.get_latest_pkg_layout(pref)
        assert os.listdir(pref_layout.metadata()) == ["logs"]
        assert os.listdir(os.path.join(pref_layout.metadata(), "logs")) == ["mylogs.txt"]
        assert load(os.path.join(pref_layout.metadata(), "logs", "mylogs.txt")) == "some logs!!!"

    def test_metadata_logs_local(self, _client):
        c = _client
        c.run("source .")
        assert c.load("metadata/logs/src.log") == "srclog!!"
        c.run("build .")
        assert c.load("mybuild/metadata/logs/mylogs.txt") == "some logs!!!"


def test_metadata_export_pkg():
    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import save, copy

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def build(self):
                save(self, "mylogs.txt", "some logs!!!")
                copy(self, "mylogs.txt", src=self.build_folder,
                     dst=os.path.join(self.package_metadata_folder, "logs"))

            def package(self):
                copy(self, "*", src=os.path.join(self.build_folder, "metadata"),
                     dst=self.package_metadata_folder)
        """)

    c = TestClient()
    c.save({"conanfile.py": conanfile})
    c.run("build .")
    c.run("export-pkg .")
    # Test local cache looks good
    pkg_layout = c.created_layout()
    assert os.listdir(pkg_layout.metadata()) == ["logs"]
    assert os.listdir(os.path.join(pkg_layout.metadata(), "logs")) == ["mylogs.txt"]
    assert load(os.path.join(pkg_layout.metadata(), "logs", "mylogs.txt")) == "some logs!!!"



import json
import textwrap

from conan.api.model import PkgReference
from conan.test.utils.tools import TestClient


def test_android_ndk():
    """ emulates the androidndk, a single package per OS-arch, that can target any
    android architecture (not especialized binary per target)
    """
    c = TestClient()

    windows = textwrap.dedent("""\
        [settings]
        os=Windows
        arch=x86_64
        """)
    linux = textwrap.dedent("""\
        [settings]
        os=Linux
        arch=x86_64
        """)
    android = textwrap.dedent("""\
        [settings]
        os=Android
        os.api_level=14
        arch = armv7
        build_type = Release
        compiler=clang
        compiler.version=11
        compiler.libcxx=c++_shared
        compiler.cppstd=14

        [tool_requires]
        androidndk/0.1
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save, copy
        class Pkg(ConanFile):
            name = "androidndk"
            version = "0.1"
            settings = "os", "arch"

            def build(self):
                save(self, "bin/ndk.compiler", f"MYNDK-{self.settings.os}-{self.settings.arch} exe!")

            def package(self):
                copy(self, "*", src=self.build_folder, dst=self.package_folder)

            def package_info(self):
                arch = self.settings_target.arch
                self.cpp_info.libs = [f"libndklib-{arch}"]
                self.buildenv_info.define("MY_ANDROID_ARCH", f"android-{arch}")
        """)
    test = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load
        from conan.tools.env import VirtualBuildEnv
        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"

            def generate(self):
                ndk = self.dependencies.build["androidndk"]
                self.output.info(f"NDK LIBS: {ndk.cpp_info.libs}!!!")
                compiler = os.path.join(ndk.package_folder, "bin/ndk.compiler")
                self.output.info(load(self, compiler))
                env = VirtualBuildEnv(self).vars()
                self.output.info(f"MY-VAR: {env.get('MY_ANDROID_ARCH')}")

            def test(self):
                pass
        """)

    c.save({"conanfile.py": conanfile,
            "windows": windows,
            "linux": linux,
            "android": android})

    # IMPORTANT: The consumption via test_package define the relation. If not existing
    # I need to pass --build-require

    # Creating the NDK packages for Windows, Linux
    c.run("create . -pr:b=windows -pr:h=android --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("522dcea5982a3f8a5b624c16477e47195da2f84f", "Build")},
                           build=True)
    # The same NDK can be used for different architectures, this should not require a new NDK build
    c.run("create . -pr:b=windows -pr:h=android -s:h arch=armv8 --build=missing  --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("522dcea5982a3f8a5b624c16477e47195da2f84f", "Cache")},
                           build=True)
    assert "androidndk/0.1: Already installed!" in c.out
    # But a different build architecture is a different NDK executable
    c.run("create . -pr:b=windows -s:b arch=x86 -pr:h=android --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("c11e463c49652ba9c5adc62573ee49f966bd8417", "Build")},
                           build=True)
    assert "androidndk/0.1: Calling build()" in c.out
    # But a different build OS is a different NDK executable
    c.run("create . -pr:b=linux -pr:h=android  --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("63fead0844576fc02943e16909f08fcdddd6f44b", "Build")},
                           build=True)
    assert "androidndk/0.1: Calling build()" in c.out

    # IMPORTANT: The consumption via test_package allows specifying the type of requires
    # in this case: None, as this is intended to be injected via profile [tool_requires]
    # can be tested like that
    c.run("remove * -c")
    c.save({"test_package/conanfile.py": test})

    # Creating the NDK packages for Windows, Linux
    c.run("create . -pr:b=windows -pr:h=android --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("522dcea5982a3f8a5b624c16477e47195da2f84f", "Build")},
                           build=True)
    assert "androidndk/0.1 (test package): NDK LIBS: ['libndklib-armv7']!!!" in c.out
    assert "androidndk/0.1 (test package): MYNDK-Windows-x86_64 exe!" in c.out
    assert "androidndk/0.1 (test package): MY-VAR: android-armv7" in c.out
    # The same NDK can be used for different architectures, this should not require a new NDK build
    c.run("create . -pr:b=windows -pr:h=android -s:h arch=armv8 --build=missing --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("522dcea5982a3f8a5b624c16477e47195da2f84f", "Cache")},
                           build=True)
    assert "androidndk/0.1: Already installed!" in c.out
    assert "androidndk/0.1 (test package): NDK LIBS: ['libndklib-armv8']!!!" in c.out
    assert "androidndk/0.1 (test package): MYNDK-Windows-x86_64 exe!" in c.out
    assert "androidndk/0.1 (test package): MY-VAR: android-armv8" in c.out

    # But a different build architecture is a different NDK executable
    c.run("create . -pr:b=windows -s:b arch=x86 -pr:h=android --build=missing --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("c11e463c49652ba9c5adc62573ee49f966bd8417", "Build")},
                           build=True)
    assert "androidndk/0.1: Calling build()" in c.out
    assert "androidndk/0.1 (test package): NDK LIBS: ['libndklib-armv7']!!!" in c.out
    assert "androidndk/0.1 (test package): MYNDK-Windows-x86 exe!" in c.out
    assert "androidndk/0.1 (test package): MY-VAR: android-armv7" in c.out

    # But a different build OS is a different NDK executable
    c.run("create . -pr:b=linux -pr:h=android --build=missing --build-require")
    c.assert_listed_binary({"androidndk/0.1": ("63fead0844576fc02943e16909f08fcdddd6f44b", "Build")},
                           build=True)
    assert "androidndk/0.1: Calling build()" in c.out
    assert "androidndk/0.1 (test package): NDK LIBS: ['libndklib-armv7']!!!" in c.out
    assert "androidndk/0.1 (test package): MYNDK-Linux-x86_64 exe!" in c.out
    assert "androidndk/0.1 (test package): MY-VAR: android-armv7" in c.out

    # Now any other package can use it
    c.save({"conanfile.py": test,
            "windows": windows,
            "linux": linux,
            "android": android}, clean_first=True)
    c.run("install . -pr:b=windows -pr:h=android")
    c.assert_listed_binary({"androidndk/0.1": ("522dcea5982a3f8a5b624c16477e47195da2f84f", "Cache")},
                           build=True)
    assert "conanfile.py: NDK LIBS: ['libndklib-armv7']!!!" in c.out
    assert "conanfile.py: MYNDK-Windows-x86_64 exe!" in c.out
    # And build on the fly the NDK if not binary exists
    c.run("install . -pr:b=linux -s:b arch=x86 -pr:h=android -s:h arch=armv8 --build=missing")
    c.assert_listed_binary({"androidndk/0.1": ("f14494c6e4810a63f050d6f5f37e9776ed48d3c9", "Build")},
                           build=True)
    assert "conanfile.py: NDK LIBS: ['libndklib-armv8']!!!" in c.out
    assert "conanfile.py: MYNDK-Linux-x86 exe!" in c.out
    assert "conanfile.py: MY-VAR: android-armv8" in c.out


def test_libcxx():
    """ emulates a package for libcxx, containing only a library to link with
    """
    c = TestClient()
    macos = textwrap.dedent("""
        [settings]
        os=Macos
        arch = x86_64
        build_type = Release
        compiler=apple-clang
        compiler.version=12.0
        compiler.cppstd=14
        compiler.libcxx=libc++
        """)
    ios = textwrap.dedent("""\
        [settings]
        os=iOS
        os.version = 14.3
        os.sdk = iphoneos
        arch = armv7
        build_type = Release
        compiler=apple-clang
        compiler.version=11.0
        compiler.cppstd=14
        compiler.libcxx=libc++

        [tool_requires]
        libcxx/0.1
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save, copy
        class Pkg(ConanFile):
            name = "libcxx"
            version = "0.1"
            settings = "os", "arch", "compiler", "build_type"

            def build(self):
                # HERE IT MUST USE THE SETTINGS-TARGET for CREATING THE BINARIES
                arch = self.settings_target.arch
                os_ = self.settings_target.os
                phone_sdk = self.settings_target.get_safe("os.sdk") or ""
                save(self, f"lib/libcxx-{arch}", f"libcxx{phone_sdk}-{os_}-{arch}!")

            def package(self):
                copy(self, "*", src=self.build_folder, dst=self.package_folder)

            def package_info(self):
                arch = self.settings_target.arch
                self.cpp_info.libs = [f"libcxx-{arch}"]

            def package_id(self):
                self.info.settings.clear()
                self.info.settings_target = self.settings_target
        """)
    test = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load
        class Pkg(ConanFile):
            settings = "os", "arch", "compiler", "build_type"

            def generate(self):
                libcxx = self.dependencies.build["libcxx"]
                libcxx_lib = libcxx.cpp_info.libs[0]
                self.output.info(f"LIBCXX LIBS: {libcxx_lib}!!!")
                libcxx_path = os.path.join(libcxx.package_folder, "lib", libcxx_lib)
                self.output.info(load(self, libcxx_path))

            def test(self):
                pass
        """)

    c.save({"conanfile.py": conanfile,
            "test_package/conanfile.py": test,
            "macos": macos,
            "ios": ios})

    c.run("create . -pr:b=macos -pr:h=ios --build-require")
    c.assert_listed_binary({"libcxx/0.1": ("cd83035dfa71d83329e31a13b54b03fd38836815", "Build")},
                           build=True)
    assert "libcxx/0.1 (test package): LIBCXX LIBS: libcxx-armv7!!!" in c.out
    assert "libcxx/0.1 (test package): libcxxiphoneos-iOS-armv7!" in c.out

    # Same host profile should be same binary, the build profile is not factored in
    c.run("create . -pr:b=macos -s:b build_type=Debug -s:b arch=armv8 -pr:h=ios "
          "--build=missing --build-require")
    c.assert_listed_binary({"libcxx/0.1": ("cd83035dfa71d83329e31a13b54b03fd38836815", "Cache")},
                           build=True)
    assert "libcxx/0.1 (test package): LIBCXX LIBS: libcxx-armv7!!!" in c.out
    assert "libcxx/0.1 (test package): libcxxiphoneos-iOS-armv7!" in c.out

    # But every change in host, is a different binary
    c.run("create . -pr:b=macos -pr:h=ios -s:h arch=armv8 --build=missing --build-require")
    c.assert_listed_binary({"libcxx/0.1": ("af2d5e0458816f1d7e61c82315d143c20e3db2e3", "Build")},
                           build=True)
    assert "libcxx/0.1 (test package): LIBCXX LIBS: libcxx-armv8!!!" in c.out
    assert "libcxx/0.1 (test package): libcxxiphoneos-iOS-armv8!" in c.out

    # But every change in host, is a different binary
    c.run("create . -pr:b=macos -pr:h=ios -s:h arch=armv8 -s:h os.sdk=iphonesimulator "
          "--build-require")
    c.assert_listed_binary({"libcxx/0.1": ("c7e3b0fbb4fac187f59bf97e7958631b96fda2a6", "Build")},
                           build=True)
    assert "libcxx/0.1 (test package): LIBCXX LIBS: libcxx-armv8!!!" in c.out
    assert "libcxx/0.1 (test package): libcxxiphonesimulator-iOS-armv8!" in c.out

    # Now any other package can use it
    c.save({"conanfile.py": test,
            "macos": macos,
            "ios": ios}, clean_first=True)
    c.run("install . -pr:b=macos -pr:h=ios")
    c.assert_listed_binary({"libcxx/0.1": ("cd83035dfa71d83329e31a13b54b03fd38836815", "Cache")},
                           build=True)
    assert "conanfile.py: LIBCXX LIBS: libcxx-armv7!!!" in c.out
    assert "conanfile.py: libcxxiphoneos-iOS-armv7!" in c.out


def test_compiler_gcc():
    """ this is testing a gcc-like cross-compiler that needs the gcc.exe binary to compile
    and can also contain a specific libcxx for the target architecture
    """
    c = TestClient()
    # build machine
    linux = textwrap.dedent("""
        [settings]
        os=Linux
        arch = x86_64
        build_type = Release
        compiler=gcc
        compiler.version=11
        compiler.cppstd=14
        compiler.libcxx=libstdc++11
        """)
    rpi = textwrap.dedent("""\
        [settings]
        os=Linux
        arch = armv7
        build_type = Release

        [tool_requires]
        gcc/0.1
        """)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import save, copy
        class Pkg(ConanFile):
            name = "gcc"
            version = "0.1"
            settings = "os", "arch", "compiler", "build_type"

            def build(self):
                # HERE IT MUST USE THE SETTINGS-TARGET for CREATING THE LIBCXX
                # BUT SETTINGS for CREATING THE GCC.EXE
                arch = self.settings_target.arch
                os_ = self.settings_target.os
                save(self, f"lib/libcxx-{arch}", f"libcxx-{os_}-{arch}!")
                save(self, "bin/gcc", f"gcc-{self.settings.os}-{self.settings.arch}")

            def package(self):
                copy(self, "*", src=self.build_folder, dst=self.package_folder)

            def package_info(self):
                arch = self.settings_target.arch
                self.cpp_info.libs = [f"libcxx-{arch}"]

            def package_id(self):
                self.info.settings_target = self.settings_target
        """)
    test = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import load
        class Pkg(ConanFile):
            settings = "os", "arch", "build_type"

            def generate(self):
                gcc = self.dependencies.build["gcc"]
                libcxx_lib = gcc.cpp_info.libs[0]
                self.output.info(f"LIBCXX LIBS: {libcxx_lib}!!!")
                libcxx_path = os.path.join(gcc.package_folder, "lib", libcxx_lib)
                self.output.info(load(self, libcxx_path))
                gcc_path = os.path.join(gcc.package_folder, "bin/gcc")
                self.output.info(load(self, gcc_path))

            def test(self):
                pass
        """)

    c.save({"conanfile.py": conanfile,
            "test_package/conanfile.py": test,
            "linux": linux,
            "rpi": rpi})

    c.run("create . -pr:b=linux -pr:h=rpi --build-require")
    c.assert_listed_binary({"gcc/0.1": ("6190ea2804cd4777609ec7174ccfdee22c6318c3", "Build")},
                           build=True)
    assert "gcc/0.1 (test package): LIBCXX LIBS: libcxx-armv7!!!" in c.out
    assert "gcc/0.1 (test package): libcxx-Linux-armv7!" in c.out
    assert "gcc/0.1 (test package): gcc-Linux-x86_64" in c.out

    # Same host profile, but different build profile is a different binary
    c.run("create . -pr:b=linux  -s:b os=Windows -s:b arch=armv8 -pr:h=rpi --build-require")
    c.assert_listed_binary({"gcc/0.1": ("687fdc5f43017300a98643948869b4c5560ca82c", "Build")},
                           build=True)
    assert "gcc/0.1 (test package): LIBCXX LIBS: libcxx-armv7!!!" in c.out
    assert "gcc/0.1 (test package): libcxx-Linux-armv7!" in c.out
    assert "gcc/0.1 (test package): gcc-Windows-armv8" in c.out

    # Same build but different host is also a new binary
    c.run("create . -pr:b=linux -pr:h=rpi -s:h arch=armv8 --build=missing --build-require")
    c.assert_listed_binary({"gcc/0.1": ("0521de9a6b94083bd47474a51570a3b856b77406", "Build")},
                           build=True)
    assert "gcc/0.1 (test package): LIBCXX LIBS: libcxx-armv8!!!" in c.out
    assert "gcc/0.1 (test package): libcxx-Linux-armv8!" in c.out
    assert "gcc/0.1 (test package): gcc-Linux-x86_64" in c.out
    # check the list packages
    c.run("list gcc/0.1:* --format=json", redirect_stdout="packages.json")
    pkgs_json = json.loads(c.load("packages.json"))
    pref = PkgReference.loads("gcc/0.1#a8d725d9988de633accf410fb04cd162:6190ea2804cd4777609ec7174ccfdee22c6318c3")
    settings_target = {
        "arch": "armv7",
        "build_type": "Release",
        "os": "Linux"
    }
    revision_dict = pkgs_json["Local Cache"]["gcc/0.1"]["revisions"][pref.ref.revision]
    assert settings_target == revision_dict["packages"][pref.package_id]["info"]["settings_target"]



import os
import re
import textwrap

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_component_error():
    # https://github.com/conan-io/conan/issues/12027
    c = TestClient()
    t1 = textwrap.dedent("""
        from conan import ConanFile

        class t1Conan(ConanFile):
            name = "t1"
            version = "0.1.0"
            package_type = "static-library"

            def package_info(self):
                self.cpp_info.components["comp1"].set_property("cmake_target_name", "t1::comp1")
                self.cpp_info.components["comp2"].set_property("cmake_target_name", "t1::comp2")
        """)
    t2 = textwrap.dedent("""
        from conan import ConanFile

        class t2Conan(ConanFile):
            name = "t2"
            version = "0.1.0"
            requires = "t1/0.1.0"
            package_type = "shared-library"

            def package_info(self):
                self.cpp_info.requires.append("t1::comp1")
        """)
    t3 = textwrap.dedent("""
        from conan import ConanFile

        class t3Conan(ConanFile):
            name = "t3"
            version = "0.1.0"
            requires = "t2/0.1.0"
            package_type = "application"
            generators = "CMakeDeps"
            settings = "os", "arch", "compiler", "build_type"
        """)

    c.save({"t1/conanfile.py": t1,
            "t2/conanfile.py": t2,
            "t3/conanfile.py": t3})
    c.run("create t1")
    c.run("create t2")
    c.run("install t3")

    arch = c.get_default_host_profile().settings['arch']
    assert 'list(APPEND t2_FIND_DEPENDENCY_NAMES )' in c.load(f"t3/t2-release-{arch}-data.cmake")
    assert not os.path.exists(os.path.join(c.current_folder, "t3/t1-config.cmake"))


def test_verify_get_property_check_type():
    c = TestClient(light=True)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class HelloConan(ConanFile):
            name = "hello"
            version = "0.1"
            def package_info(self):
                self.cpp_info.set_property("test_property", "foo")
                self.cpp_info.get_property("test_property", check_type=list)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .", assert_error=True)
    assert 'The expected type for test_property is "list", but "str" was found' in c.out


@pytest.mark.parametrize("component", [True, False])
def test_unused_requirement(component):
    """ Requires should include all listed requirements
        This error is known when creating the package if the requirement is consumed.
    """

    t = TestClient(light=True)
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        class Consumer(ConanFile):
            name = "wrong"
            version = "version"
            requires = "top/version", "top2/version"
            def package_info(self):
                self.cpp_info{'.components["foo"]' if component else ''}.requires = ["top::other"]
    """)
    t.save({"top/conanfile.py": GenConanfile().with_package_info({"components": {"cmp1": {"libs": ["top_cmp1"]}}}),
            "conanfile.py": conanfile})
    t.run('create top --name=top --version=version')
    t.run('create top --name=top2 --version=version')
    t.run('create .', assert_error=True)
    assert "ERROR: wrong/version: package_info(): The direct dependency 'top2' is not used " \
           "by any '(cpp_info/components).requires'." in t.out


def test_unused_requirement_not_propagated():
    # https://github.com/conan-io/conan/issues/19026
    t = TestClient()
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        class Consumer(ConanFile):
            name = "pkg"
            version = "0.1"
            requires = "header/0.1", "lib/0.1"

            def package_info(self):
                self.cpp_info.requires = ["lib::lib", "header::header"]
    """)
    t.save({"header/conanfile.py": GenConanfile("header", "0.1").with_package_type("header-library"),
            "lib/conanfile.py": GenConanfile("lib", "0.1").with_package_type("static-library"),
            "pkg/conanfile.py": conanfile,
            "app/conanfile.py": GenConanfile().with_requires("pkg/0.1")})
    t.run('create header')
    t.run('create lib')
    t.run('create pkg')
    t.run("install app")
    assert re.search(r"Skipped binaries(\s*)header/0.1", t.out)


@pytest.mark.parametrize("component", [True, False])
def test_wrong_requirement(component):
    """ If we require a wrong requirement, we get a meaninful error.
        This error is known when creating the package if the requirement is not there.
    """
    t = TestClient(light=True)
    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        class Consumer(ConanFile):
            name = "wrong"
            version = "version"
            requires = "top/version"
            def package_info(self):
                self.cpp_info{'.components["foo"]' if component else ''}.requires =  ["top::cmp1", "other::other"]
    """)
    t.save({"top/conanfile.py": GenConanfile().with_package_info({"components": {"cmp1": {"libs": ["top_cmp1"]}}}),
            "conanfile.py": conanfile})
    t.run('create top --name=top --version=version')
    t.run('create .', assert_error=True)
    assert "ERROR: wrong/version: package_info(): There are '(cpp_info/components).requires' " \
           "that includes package 'other::', but such package is not a a direct requirement " \
           "of the recipe" in t.out


@pytest.mark.parametrize("component", [True, False])
def test_missing_internal(component):
    consumer = textwrap.dedent(f"""
        from conan import ConanFile
        class Recipe(ConanFile):
            def package_info(self):
                self.cpp_info{'.components["foo"]' if component else ''}.requires = ["other", "another"]
                self.cpp_info{'.components["bar"]' if component else ''}.requires = ["other", "another"]
    """)
    t = TestClient(light=True)
    t.save({'conanfile.py': consumer})
    t.run('create . --name=wrong --version=version', assert_error=True)
    assert "ERROR: wrong/version: package_info(): There are '(cpp_info/components).requires' " \
           "to other internal components that are not defined: ['other', 'another']" in t.out


def test_unused_tool_requirement():
    """ Requires should include all listed requirements
        This error is known when creating the package if the requirement is consumed.
    """
    top_conanfile = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):

            def package_info(self):
                self.cpp_info.components["cmp1"].libs = ["top_cmp1"]
                self.cpp_info.components["cmp2"].libs = ["top_cmp2"]
        """)
    consumer = textwrap.dedent("""
        from conan import ConanFile

        class Recipe(ConanFile):
            requires = "top/version"
            tool_requires = "top2/version"
            def package_info(self):
                self.cpp_info.requires = ["top::other"]
    """)
    t = TestClient()
    t.save({'top.py': top_conanfile, 'consumer.py': consumer})
    t.run('create top.py --name=top --version=version')
    t.run('create top.py --name=top2 --version=version')
    t.run('create consumer.py --name=wrong --version=version')
    # This runs without crashing, because it is not chcking that top::other doesn't exist


def test_component_double_colon_error_message():
    c = TestClient()
    t2 = textwrap.dedent("""
        from conan import ConanFile

        class t2Conan(ConanFile):
            name = "t2"
            version = "0.1.0"
            requires = "t1/0.1.0"
            def package_info(self):
                self.cpp_info.requires.append("t1::comp1::other")
        """)

    c.save({"t1/conanfile.py": GenConanfile("t1", "0.1.0"),
            "t2/conanfile.py": t2,
            "t2/test_package/conanfile.py": GenConanfile().with_settings("build_type")
                                                          .with_generator("CMakeDeps")
                                                          .with_test("pass")})
    c.run("create t1")
    c.run("create t2", assert_error=True)
    assert "Component 't1::comp1::other' not found in 't1' package requirement" in c.out



import platform

import pytest

from conan.test.utils.mocks import ConanFileMock
from conan.tools.google import Bazel
from conan.tools.google.bazeldeps import _relativize_path


@pytest.mark.skipif(platform.system() == "Windows", reason="Remove this skip for Conan 2.x"
                                                           "Needs conanfile.commands")
def test_bazel_command_with_empty_config():
    conanfile = ConanFileMock()
    bazel = Bazel(conanfile)
    bazel.build(target='//test:label')
    # Uncomment Conan 2.x
    assert 'bazel build //test:label' in conanfile.commands


@pytest.mark.skipif(platform.system() == "Windows", reason="Remove this skip for Conan 2.x."
                                                           "Needs conanfile.commands")
def test_bazel_command_with_config_values():
    conanfile = ConanFileMock()
    conanfile.conf.define("tools.google.bazel:configs", ["config", "config2"])
    conanfile.conf.define("tools.google.bazel:bazelrc_path", ["/path/to/bazelrc"])
    bazel = Bazel(conanfile)
    bazel.build(target='//test:label')
    commands = conanfile.commands
    assert "bazel --bazelrc=/path/to/bazelrc build " \
           "--config=config --config=config2 //test:label" in commands
    assert "bazel --bazelrc=/path/to/bazelrc clean" in commands


@pytest.mark.parametrize("path, pattern, expected", [
    ("", "./", ""),
    ("./", "", "./"),
    ("/my/path/", "", "/my/path/"),
    ("\\my\\path\\", "", "\\my\\path\\"),
    ("/my/path/absolute", ".*/path", "absolute"),
    ("/my/path/absolute", "/my/path", "absolute"),
    ("\\my\\path\\absolute", "/my/path", "absolute"),
    ("/my/./path/absolute/", "/my/./path", "absolute"),
    ("/my/./path/absolute/", "/my/./path/absolute/", "./"),
    ("././my/path/absolute/././", "./", "my/path/absolute"),
    ("C:\\my\\path\\absolute\\with\\folder", "C:\\", "my/path/absolute/with/folder"),
    ("C:\\my\\path\\absolute\\with\\folder", ".*/absolute", "with/folder"),
    ("C:\\my\\path\\myabsolute\\with\\folder", ".*/absolute", "C:\\my\\path\\myabsolute\\with\\folder"),
])
def test_bazeldeps_relativize_path(path, pattern, expected):
    assert _relativize_path(path, pattern) == expected



import platform
import textwrap

import pytest

from conan.test.utils.tools import TestClient

new_value = "will_break_next"


@pytest.mark.skipif(platform.system() != "Darwin", reason="Only OSX")
def test_package_framework_needs_location():
    conanfile = textwrap.dedent(f"""
    import os
    from conan import ConanFile

    class MyFramework(ConanFile):
        name = "frame"
        version = "1.0"
        settings = "os", "arch", "compiler", "build_type"
        package_type = 'static-library'

        def package_info(self):
            self.cpp_info.type = 'static-library'
            self.cpp_info.package_framework = "MyFramework"
    """)
    test_conanfile = textwrap.dedent("""
    import os
    from conan import ConanFile

    class LibTestConan(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "CMakeConfigDeps", "CMakeToolchain"

        def requirements(self):
            self.requires(self.tested_reference_str)
    """)
    client = TestClient()
    client.save({
        'test_package/conanfile.py': test_conanfile,
        'conanfile.py': conanfile
    })
    client.run(f"create . -c tools.cmake.cmakedeps:new={new_value}", assert_error=True)
    assert "Error in generator 'CMakeConfigDeps': cpp_info.location missing for framework MyFramework" in client.out


def test_framework_only_component_generates_target():
    conanfile = textwrap.dedent(f"""
    import os
    from conan import ConanFile

    class MyFramework(ConanFile):
        name = "frame"
        version = "1.0"
        settings = "os", "arch", "compiler", "build_type"

        def package_info(self):
            self.cpp_info.frameworks = ["MyFramework"]
            self.cpp_info.libdirs = []
            self.cpp_info.includedirs = []
    """)

    tc = TestClient()
    tc.save({"conanfile.py": conanfile})
    tc.run("create .")
    tc.run(f"install --requires=frame/1.0 -g CMakeConfigDeps -c=tools.cmake.cmakedeps:new={new_value}")
    targets = tc.load("frame-Targets-release.cmake")
    assert "add_library(frame::frame INTERFACE IMPORTED)" in targets



import os

from conan.tools.build import save_toolchain_args
from conan.tools.gnu import Autotools
from conan import ConanFile
from conan.internal.model.conf import Conf
from test.unittests.util.tools_test import RunnerMock
from conan.test.utils.mocks import MockSettings
from conan.test.utils.test_files import temp_folder


def test_configure_arguments():
    tmp = temp_folder()
    os.chdir(tmp)
    save_toolchain_args({
        "configure_args": "my_configure_args",
        "make_args": "my_make_args"}
    )
    runner = RunnerMock()
    conanfile = ConanFile()
    conanfile.run = runner
    conanfile.settings = MockSettings({"os": "Linux"})
    conanfile.settings_build = conanfile.settings
    conanfile.folders.set_base_source(tmp)
    conanfile.conf = Conf()
    conanfile.conf.define("tools.gnu:make_program", "my_make")
    conanfile.conf.define("tools.build:jobs", 23)
    ab = Autotools(conanfile)
    ab.configure()
    assert "configure\" my_configure_args" in runner.command_called

    ab = Autotools(conanfile)
    ab.make()
    assert "my_make my_make_args -j23" == runner.command_called

    # test install target argument

    ab.install()
    assert 'my_make install my_make_args DESTDIR=None -j23' == runner.command_called

    ab.install(target="install_other")
    assert 'my_make install_other my_make_args DESTDIR=None -j23' == runner.command_called

    for make_args in ["my_make_args", ""]:

        save_toolchain_args({
            "configure_args": "my_configure_args",
            "make_args": f"{make_args}"}
        )

        ab = Autotools(conanfile)

        make_args_str = f" {make_args}" if make_args else ""

        ab.make(args=["-j1"])
        assert "-j23" not in runner.command_called
        assert f"my_make{make_args_str} -j1" == runner.command_called

        ab.install(args=["-j1"])
        assert "-j23" not in runner.command_called
        assert f"my_make install{make_args_str} DESTDIR=None -j1" == runner.command_called

        ab.install(args=["DESTDIR=whatever", "-j1"])
        assert "-j23" not in runner.command_called
        assert f"my_make install{make_args_str} DESTDIR=whatever -j1" == runner.command_called

        ab.install(args=["DESTDIR=whatever", "-arg1 -j1 -arg2"])
        assert "-j23" not in runner.command_called
        assert f"my_make install{make_args_str} DESTDIR=whatever -arg1 -j1 -arg2" == runner.command_called

        # check that we don't detect -j in an argument as number of jobs
        ab.install(args=["DESTDIR=/user/smith-john/whatever"])
        assert f"my_make install{make_args_str} DESTDIR=/user/smith-john/whatever -j23" == runner.command_called

        # check that we don't detect -j in an argument as number of jobs
        ab.install(args=["DESTDIR=/user/smith-j47/whatever"])
        assert f"my_make install{make_args_str} DESTDIR=/user/smith-j47/whatever -j23" == runner.command_called



from conan.test.utils.tools import TestClient

base_conanfile = '''
from conan import ConanFile

class TestSystemReqs(ConanFile):
    name = "test"
    version = "0.1"

    def system_requirements(self):
        self.output.info("*+Running system requirements+*")
'''


class TestSystemReqs:

    def test_force_system_reqs_rerun(self):
        client = TestClient()
        client.save({'conanfile.py': base_conanfile})
        client.run("create . ")
        assert "*+Running system requirements+*" in client.out
        client.run("install --requires=test/0.1")
        assert "*+Running system requirements+*" in client.out

    def test_local_system_requirements(self):
        client = TestClient()
        client.save({'conanfile.py': base_conanfile})
        client.run("install .")
        assert "*+Running system requirements+*" in client.out



import json
import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.api.model import RecipeReference
from conan.internal.util.files import load


def _metadata(c, ref):
    pref = c.get_latest_package_reference(RecipeReference.loads(ref))
    return c.get_latest_pkg_layout(pref).metadata()


def test_subgraph_reports():
    c = TestClient()
    subgraph_hook = textwrap.dedent("""\
        import os, json
        from conan.tools.files import save
        from conan.internal.model.lockfile import Lockfile
        def post_package(conanfile):
            subgraph = conanfile.subgraph
            save(conanfile, os.path.join(conanfile.package_metadata_folder, f"conangraph.json"),
                 json.dumps(subgraph.serialize(), indent=2))
            save(conanfile, os.path.join(conanfile.package_metadata_folder, f"conan.lock"),
                 Lockfile(subgraph).dumps())
        """)

    c.save_home({"extensions/hooks/subgraph_hook/hook_subgraph.py": subgraph_hook})
    c.save({"dep/conanfile.py": GenConanfile("dep", "0.1"),
            "pkg/conanfile.py": GenConanfile("pkg", "0.1").with_requirement("dep/0.1"),
            "app/conanfile.py": GenConanfile("app", "0.1").with_requirement("pkg/0.1")})
    c.run("export dep")
    c.run("export pkg")
    # app -> pkg -> dep
    c.run("create app --build=missing --format=json")

    app_graph = json.loads(load(os.path.join(_metadata(c, "app/0.1"), "conangraph.json")))
    pkg_graph = json.loads(load(os.path.join(_metadata(c, "pkg/0.1"), "conangraph.json")))
    dep_graph = json.loads(load(os.path.join(_metadata(c, "dep/0.1"), "conangraph.json")))

    app_lock = json.loads(load(os.path.join(_metadata(c, "app/0.1"), "conan.lock")))
    pkg_lock = json.loads(load(os.path.join(_metadata(c, "pkg/0.1"), "conan.lock")))
    dep_lock = json.loads(load(os.path.join(_metadata(c, "dep/0.1"), "conan.lock")))

    assert len(app_graph["nodes"]) == len(app_lock["requires"])
    assert len(pkg_graph["nodes"]) == len(pkg_lock["requires"])
    assert len(dep_graph["nodes"]) == len(dep_lock["requires"])



import os

import pytest

from conan.test.utils.tools import TestClient


@pytest.mark.tool("ninja")
@pytest.mark.tool("meson")
@pytest.mark.tool("pkg_config")
def test_meson_lib_template():
    # Identical to def test_cmake_lib_template(), but for Meson
    client = TestClient(path_with_spaces=False)
    client.run("new meson_lib -d name=hello -d version=0.1")

    # Local flow works
    client.run("install .")
    client.run("build .")
    client.run("export-pkg . --name=hello --version=0.1")
    package_folder = client.created_layout().package()
    assert os.path.exists(os.path.join(package_folder, "include", "hello.h"))

    # Create works
    client.run("create .")
    assert "hello/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "hello/0.1: Hello World Debug!" in client.out

    # Create + shared works
    client.run("create . -o hello/*:shared=True")
    assert "hello/0.1: Hello World Release!" in client.out


@pytest.mark.tool("ninja")
@pytest.mark.tool("meson")
def test_meson_exe_template():
    client = TestClient(path_with_spaces=False)
    client.run("new meson_exe -d name=greet -d version=0.1")
    # Local flow works
    client.run("install .")
    client.run("build .")

    # Create works
    client.run("create .")
    assert "greet/0.1: Hello World Release!" in client.out

    client.run("create . -s build_type=Debug")
    assert "greet/0.1: Hello World Debug!" in client.out



import textwrap
from collections import OrderedDict
from unittest.mock import patch, Mock

import pytest

from conan.internal.errors import ConanConnectionError
from conan.errors import ConanException
from conan.api.model import RecipeReference
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient, TestServer


class TestSearch:

    @pytest.fixture
    def remotes(self):
        self.servers = OrderedDict()
        self.servers["remote1"] = TestServer(server_capabilities=[])
        self.servers["remote2"] = TestServer(server_capabilities=[])

        self.client = TestClient(servers=self.servers)

    def test_search_no_params(self):
        self.servers = OrderedDict()
        self.client = TestClient(servers=self.servers)

        self.client.run("search", assert_error=True)
        assert "error: the following arguments are required: reference" in self.client.out

    def test_search_no_matching_recipes(self, remotes):
        expected_output = ("Connecting to remote 'remote1' anonymously\n"
                           "Connecting to remote 'remote2' anonymously\n"
                           "remote1\n"
                           "  ERROR: Recipe 'whatever' not found\n"
                           "remote2\n"
                           "  ERROR: Recipe 'whatever' not found\n")

        self.client.run("search whatever")
        assert expected_output == self.client.out

    def test_search_no_configured_remotes(self):
        self.servers = OrderedDict()
        self.client = TestClient(servers=self.servers)

        self.client.run("search whatever", assert_error=True)
        assert "There are no remotes to search from" in self.client.out

    def test_search_disabled_remote(self, remotes):
        self.client.run("remote disable remote1")
        self.client.run("search whatever -r remote1", assert_error=True)
        expected_output = "ERROR: Remote 'remote1' can't be found or is disabled"
        assert expected_output in self.client.out


class TestRemotes:

    @pytest.fixture(autouse=True)
    def _setup(self):
        self.servers = OrderedDict()
        self.users = {}
        self.client = TestClient()

    def _add_remote(self, remote_name):
        self.servers[remote_name] = TestServer(users={"user": "passwd"})
        self.users[remote_name] = [("user", "passwd")]
        self.client = TestClient(servers=self.servers, inputs=["user", "passwd"])

    def _add_recipe(self, remote, reference):
        conanfile = textwrap.dedent("""
            from conan import ConanFile
            class MyLib(ConanFile):
                pass
            """)

        self.client.save({'conanfile.py': conanfile})
        reference = RecipeReference.loads(str(reference))
        self.client.run(f"export . --name={reference.name} --version={reference.version} --user={reference.user} --channel={reference.channel}")
        self.client.run("upload --force -r {} {}".format(remote, reference))

    @pytest.mark.parametrize("exc,output", [
        (ConanConnectionError("Review your network!"),
         "ERROR: Review your network!"),
        (ConanException("Boom!"), "ERROR: Boom!")
    ])
    def test_search_remote_errors_but_no_raising_exceptions(self, exc, output):
        self._add_remote("remote1")
        self._add_remote("remote2")
        with patch("conan.api.subapi.list._search_recipes", new=Mock(side_effect=exc)):
            self.client.run("search whatever")
        expected_output = textwrap.dedent(f"""\
        remote1
          {output}
        remote2
          {output}
        """)
        assert expected_output == self.client.out

    def test_no_remotes(self):
        self.client.run("search something", assert_error=True)
        expected_output = "There are no remotes to search from"
        assert expected_output in self.client.out

    def test_search_by_name(self):
        remote_name = "remote1"
        recipe_name = "test_recipe/1.0.0@user/channel"
        self._add_remote(remote_name)
        self._add_recipe(remote_name, recipe_name)

        self.client.run("search -r {} {}".format(remote_name, "test_recipe"))

        expected_output = (
            "remote1\n"
            "  test_recipe\n"
            "    {}\n".format(recipe_name)
        )

        assert expected_output in self.client.out

    def test_search_in_all_remotes(self):
        remote1 = "remote1"
        remote2 = "remote2"

        remote1_recipe1 = "test_recipe/1.0.0@user/channel"
        remote1_recipe2 = "test_recipe/1.1.0@user/channel"
        remote2_recipe1 = "test_recipe/2.0.0@user/channel"
        remote2_recipe2 = "test_recipe/2.1.0@user/channel"

        expected_output = (
            "remote1\n"
            "  test_recipe\n"
            "    test_recipe/1.0.0@user/channel\n"
            "    test_recipe/1.1.0@user/channel\n"
            "remote2\n"
            "  test_recipe\n"
            "    test_recipe/2.0.0@user/channel\n"
            "    test_recipe/2.1.0@user/channel\n"
        )

        self._add_remote(remote1)
        self._add_recipe(remote1, remote1_recipe1)
        self._add_recipe(remote1, remote1_recipe2)

        self._add_remote(remote2)
        self._add_recipe(remote2, remote2_recipe1)
        self._add_recipe(remote2, remote2_recipe2)

        self.client.run("search test_recipe")
        assert expected_output in self.client.out

    def test_search_in_one_remote(self):
        remote1 = "remote1"
        remote2 = "remote2"

        remote1_recipe1 = "test_recipe/1.0.0@user/channel"
        remote1_recipe2 = "test_recipe/1.1.0@user/channel"
        remote2_recipe1 = "test_recipe/2.0.0@user/channel"
        remote2_recipe2 = "test_recipe/2.1.0@user/channel"

        expected_output = (
            "remote1\n"
            "  test_recipe\n"
            "    test_recipe/1.0.0@user/channel\n"
            "    test_recipe/1.1.0@user/channel\n"
        )

        self._add_remote(remote1)
        self._add_recipe(remote1, remote1_recipe1)
        self._add_recipe(remote1, remote1_recipe2)

        self._add_remote(remote2)
        self._add_recipe(remote2, remote2_recipe1)
        self._add_recipe(remote2, remote2_recipe2)

        self.client.run("search -r remote1 test_recipe")
        assert expected_output in self.client.out

    def test_search_package_found_in_one_remote(self):

        remote1 = "remote1"
        remote2 = "remote2"

        remote1_recipe1 = "test_recipe/1.0.0@user/channel"
        remote1_recipe2 = "test_recipe/1.1.0@user/channel"
        remote2_recipe1 = "another_recipe/2.0.0@user/channel"
        remote2_recipe2 = "another_recipe/2.1.0@user/channel"

        expected_output = (
            "remote1\n"
            "  test_recipe\n"
            "    test_recipe/1.0.0@user/channel\n"
            "    test_recipe/1.1.0@user/channel\n"
            "remote2\n"
            "  ERROR: Recipe 'test_recipe' not found\n"
        )

        self._add_remote(remote1)
        self._add_recipe(remote1, remote1_recipe1)
        self._add_recipe(remote1, remote1_recipe2)

        self._add_remote(remote2)
        self._add_recipe(remote2, remote2_recipe1)
        self._add_recipe(remote2, remote2_recipe2)

        self.client.run("search test_recipe")
        assert expected_output in self.client.out

    def test_search_in_missing_remote(self):
        remote1 = "remote1"

        remote1_recipe1 = "test_recipe/1.0.0@user/channel"
        remote1_recipe2 = "test_recipe/1.1.0@user/channel"

        self._add_remote(remote1)
        self._add_recipe(remote1, remote1_recipe1)
        self._add_recipe(remote1, remote1_recipe2)

        self.client.run("search -r wrong_remote test_recipe", assert_error=True)
        expected_output = "ERROR: Remote 'wrong_remote' can't be found or is disabled"
        assert expected_output in self.client.out

    def test_search_wildcard(self):
        remote1 = "remote1"

        remote1_recipe1 = "test_recipe/1.0.0@user/channel"
        remote1_recipe2 = "test_recipe/1.1.0@user/channel"
        remote1_recipe3 = "test_another/2.1.0@user/channel"
        remote1_recipe4 = "test_another/4.1.0@user/channel"

        expected_output = (
            "remote1\n"
            "  test_another\n"
            "    test_another/2.1.0@user/channel\n"
            "    test_another/4.1.0@user/channel\n"
            "  test_recipe\n"
            "    test_recipe/1.0.0@user/channel\n"
            "    test_recipe/1.1.0@user/channel\n"
        )

        self._add_remote(remote1)
        self._add_recipe(remote1, remote1_recipe1)
        self._add_recipe(remote1, remote1_recipe2)
        self._add_recipe(remote1, remote1_recipe3)
        self._add_recipe(remote1, remote1_recipe4)

        self.client.run("search test_*")
        assert expected_output in self.client.out


def test_no_user_channel_error():
    # https://github.com/conan-io/conan/issues/13170
    c = TestClient(default_server_user=True)
    c.save({"conanfile.py": GenConanfile("pkg")})
    c.run("export . --version=1.0")
    c.run("export . --version=1.0 --user=user --channel=channel")
    c.run("list *")
    assert "pkg/1.0" in [s.strip() for s in str(c.out).splitlines()]
    assert "pkg/1.0@user/channel" in c.out
    # I want to list only those without user/channel
    c.run("list pkg/*@")
    assert "pkg/1.0" in c.out
    assert "user/channel" not in c.out

    # The same underlying logic is used in upload
    c.run("upload pkg/*@ -r=default -c")
    assert "Uploading recipe 'pkg/1.0" in c.out
    assert "user/channel" not in c.out

    c.run("search pkg/*@ -r=default")
    assert "pkg/1.0" in c.out
    assert "user/channel" not in c.out



from conan.internal.api.new.cmake_lib import source_cpp, source_h, test_main

conanfile_sources = """import os
from conan import ConanFile
from conan.tools.files import copy
from conan.tools.layout import basic_layout
from conan.tools.premake import PremakeDeps, PremakeToolchain, Premake


class {{package_name}}Recipe(ConanFile):
    name = "{{name}}"
    version = "{{version}}"
    package_type = "library"

    # Optional metadata
    license = "<Put the package license here>"
    author = "<Put your name here> <And your email here>"
    url = "<Package recipe repository url here, for issues about the package>"
    description = "<Description of {{ name }} package here>"
    topics = ("<Put some tag here>", "<here>", "<and here>")

    # Binary configuration
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False], "fPIC": [True, False]}
    default_options = {"shared": False, "fPIC": True}
    implements = ["auto_shared_fpic"]

    # Sources are located in the same place as this recipe, copy them to the recipe
    exports_sources = "premake5.lua", "src/*", "include/*"

    def layout(self):
        basic_layout(self)
    {% if requires is defined %}
    def requirements(self):
        {% for require in requires -%}
        self.requires("{{ require }}")
        {% endfor %}
    {%- endif %}
    {%- if tool_requires is defined %}
    def build_requirements(self):
        {% for require in tool_requires -%}
        self.tool_requires("{{ require }}")
        {% endfor %}
    {%- endif %}
    def generate(self):
        deps = PremakeDeps(self)
        deps.generate()
        tc = PremakeToolchain(self)
        tc.generate()

    def build(self):
        premake = Premake(self)
        premake.configure()
        premake.build(workspace="{{name.capitalize()}}")

    def package(self):
        copy(self, "*.h", os.path.join(self.source_folder, "include"), os.path.join(self.package_folder, "include"))

        for pattern in ("*.lib", "*.a", "*.so*", "*.dylib"):
            copy(self, pattern, os.path.join(self.build_folder, "bin"), os.path.join(self.package_folder, "lib"))
        copy(self, "*.dll", os.path.join(self.build_folder, "bin"), os.path.join(self.package_folder, "bin"))

    def package_info(self):
        self.cpp_info.libs = ["{{name}}"]
"""

premake5 = """workspace "{{name.capitalize()}}"
    configurations { "Debug", "Release" }
    fatalwarnings {"All"}
    floatingpoint "Fast"
    includedirs { ".", "src", "include" }

project "{{name}}"
    cppdialect "C++17"

    -- To let conan take control over `kind` of the libraries, DO NOT SET `kind` (StaticLib or
    -- SharedLib) in `project` block.
    -- kind "<controlled by conan>"

    language "C++"
    files { "include/*.hpp", "include/*.h", "src/*.cpp" }

    filter "configurations:Debug"
       defines { "DEBUG" }
       symbols "On"

    filter "configurations:Release"
       defines { "NDEBUG" }
       optimize "On"

"""

test_conanfile = """import os

from conan import ConanFile
from conan.tools.layout import basic_layout
from conan.tools.premake import Premake
from conan.tools.build import can_run


class {{package_name}}TestConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "PremakeDeps", "PremakeToolchain"

    def layout(self):
        basic_layout(self)

    def requirements(self):
        self.requires(self.tested_reference_str)

    def build(self):
        premake = Premake(self)
        premake.configure()
        premake.build(workspace="Example")

    def test(self):
        if can_run(self):
            cmd = os.path.join(self.cpp.build.bindir, "bin", "example")
            self.run(cmd, env="conanrun")
"""

test_premake5 = """workspace "Example"
   configurations { "Debug", "Release" }

project "example"
   kind "ConsoleApp"
   language "C++"
   files { "src/example.cpp" }

   filter "configurations:Debug"
      defines { "DEBUG" }
      symbols "On"

   filter "configurations:Release"
      defines { "NDEBUG" }
      optimize "On"
"""


premake_lib_files = {
    "conanfile.py": conanfile_sources,
    "src/{{name}}.cpp": source_cpp,
    "include/{{name}}.h": source_h,
    "premake5.lua": premake5,
    "test_package/conanfile.py": test_conanfile,
    "test_package/src/example.cpp": test_main,
    "test_package/premake5.lua": test_premake5,
}



import json
import os
import platform
import sys
import textwrap
from shutil import copy
from unittest import mock

import pytest

from conan.internal.api.uploader import compress_files
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.file_server import TestFileServer
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


def test_source_download_password():
    c = TestClient()
    file_server = TestFileServer()
    c.servers["file_server"] = file_server
    save(os.path.join(file_server.store, "myfile.txt"), "hello world!")

    server_url = file_server.fake_url

    conanfile = textwrap.dedent(f"""
        from conan import ConanFile
        from conan.tools.files import download, load
        class Pkg(ConanFile):
            def source(self):
                download(self, "{server_url}/basic-auth/myfile.txt", "myfile.txt")
                self.output.info(f"Content: {{load(self, 'myfile.txt')}}")
            """)
    c.save({"conanfile.py": conanfile})
    content = {"credentials": [{"url": server_url, "token": "password"}]}
    save(os.path.join(c.cache_folder, "source_credentials.json"), json.dumps(content))
    c.run("source .")
    assert "Content: hello world!" in c.out
    content = {"credentials": [{"url": server_url,
                                "user": "user", "password": "password"}]}
    save(os.path.join(c.cache_folder, "source_credentials.json"), json.dumps(content))
    c.run("source .")
    assert "Content: hello world!" in c.out

    content = {"credentials": [{"url": server_url, "token": "{{mytk}}"}]}
    content = "{% set mytk = 'password' %}\n" + json.dumps(content)
    save(os.path.join(c.cache_folder, "source_credentials.json"), content)
    c.run("source .")
    assert "Content: hello world!" in c.out

    # Errors loading file
    for invalid in ["",
                    "potato",
                    {"token": "mytoken"},
                    {},
                    {"url": server_url},
                    {"auth": {}},
                    {"user": "other", "password": "pass"}]:
        content = {"credentials": [invalid]}
        save(os.path.join(c.cache_folder, "source_credentials.json"), json.dumps(content))
        c.run("source .", assert_error=True)
        assert "Error loading 'source_credentials.json'" in c.out

    content = {"credentials": [{"url": server_url, "token": "mytoken2"}]}
    save(os.path.join(c.cache_folder, "source_credentials.json"), json.dumps(content))
    c.run("source .", assert_error=True)
    assert "ERROR: conanfile.py: Error in source() method, line 6" in c.out
    assert "Authentication" in c.out


def test_source_credentials_only_download():
    # https://github.com/conan-io/conan/issues/16396
    c = TestClient(default_server_user=True)
    url = c.servers["default"].fake_url

    content = {"credentials": [{"url": url, "token": "password stpaces"}]}
    save(os.path.join(c.cache_folder, "source_credentials.json"), json.dumps(content))

    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("create .")
    # add_auth should never be called for regular upload/download
    with mock.patch("conan.internal.rest.conan_requester._SourceURLCredentials.add_auth", None):
        c.run("upload * -c -r=default")
        c.run("remove * -c")
        c.run("download pkg/0.1 -r=default")


@pytest.mark.skipif(sys.version_info.minor < 12 or platform.system() == "Windows",
                    reason="Extraction filters only Python 3.12, using symlinks (not Windows)")
def test_blocked_malicius_tgz():
    folder = temp_folder()
    f = os.path.join(folder, "myfile.txt")
    save(f, "The contents")
    s = os.path.join(folder, "mylink.txt")
    os.symlink(f, s)
    tgz_path = compress_files({f: f, s: s}, "myfiles.tgz", dest_dir=folder)
    os.remove(f)

    conan_file = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.files import get
        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"
            def source(self):
                get(self, "http://fake_url/myfiles.tgz")
            """)
    client = TestClient()
    client.save({"conanfile.py": conan_file})

    with mock.patch("conan.tools.files.files.download") as mock_download:
        def download_zip(*args, **kwargs):  # noqa
            copy(tgz_path, os.getcwd())
        mock_download.side_effect = download_zip
        client.run("create . -c tools.files.unzip:filter=data", assert_error=True)
        assert "AbsoluteLinkError" in client.out
        client.save({"conanfile.py": conan_file.format("extract_filter='fully_trusted'")})
        client.run("create . ")  # Doesn't fail now
        # user conf has precedence
        client.save({"conanfile.py": conan_file.format("extract_filter='data'")})
        client.run("create . -c tools.files.unzip:filter=fully_trusted")  # Doesn't fail now
